@@ -49,14 +49,14 @@ static GdkGC *gdk_window_create_gc      (GdkDrawable     *drawable,
                                          GdkGCValuesMask  mask);
 static void   gdk_window_draw_rectangle (GdkDrawable     *drawable,
                                          GdkGC           *gc,
-                                         gint             filled,
+                                         gboolean         filled,
                                          gint             x,
                                          gint             y,
                                          gint             width,
                                          gint             height);
 static void   gdk_window_draw_arc       (GdkDrawable     *drawable,
                                          GdkGC           *gc,
-                                         gint             filled,
+                                         gboolean         filled,
                                          gint             x,
                                          gint             y,
                                          gint             width,
@@ -65,7 +65,7 @@ static void   gdk_window_draw_arc       (GdkDrawable     *drawable,
                                          gint             angle2);
 static void   gdk_window_draw_polygon   (GdkDrawable     *drawable,
                                          GdkGC           *gc,
-                                         gint             filled,
+                                         gboolean         filled,
                                          GdkPoint        *points,
                                          gint             npoints);
 static void   gdk_window_draw_text      (GdkDrawable     *drawable,
@@ -694,7 +694,7 @@ gdk_window_remove_filter (GdkWindow     *window,
 
 /**
  * gdk_screen_get_toplevel_windows:
- * @screen : The #GdkScreen where the toplevels are located.
+ * @screen: The #GdkScreen where the toplevels are located.
  * 
  * Obtains a list of all toplevel windows known to GDK on the screen @screen.
  * A toplevel window is a child of the root window (see
@@ -704,6 +704,8 @@ gdk_window_remove_filter (GdkWindow     *window,
  * its elements need not be freed.
  * 
  * Return value: list of toplevel windows, free with g_list_free()
+ *
+ * Since: 2.2
  **/
 GList *
 gdk_screen_get_toplevel_windows (GdkScreen *screen)
@@ -1257,7 +1259,7 @@ gdk_window_create_gc (GdkDrawable     *drawable,
 static void
 gdk_window_draw_rectangle (GdkDrawable *drawable,
 			   GdkGC       *gc,
-			   gint         filled,
+			   gboolean     filled,
 			   gint         x,
 			   gint         y,
 			   gint         width,
@@ -1285,7 +1287,7 @@ gdk_window_draw_rectangle (GdkDrawable *drawable,
 static void
 gdk_window_draw_arc (GdkDrawable *drawable,
 		     GdkGC       *gc,
-		     gint         filled,
+		     gboolean     filled,
 		     gint         x,
 		     gint         y,
 		     gint         width,
@@ -1316,7 +1318,7 @@ gdk_window_draw_arc (GdkDrawable *drawable,
 static void
 gdk_window_draw_polygon (GdkDrawable *drawable,
 			 GdkGC       *gc,
-			 gint         filled,
+			 gboolean     filled,
 			 GdkPoint    *points,
 			 gint         npoints)
 {
