@@ -104,7 +104,11 @@ typedef enum
   GDK_WINDOW_TYPE_HINT_NORMAL,
   GDK_WINDOW_TYPE_HINT_DIALOG,
   GDK_WINDOW_TYPE_HINT_MENU,
-  GDK_WINDOW_TYPE_HINT_TOOLBAR
+  GDK_WINDOW_TYPE_HINT_TOOLBAR,
+  GDK_WINDOW_TYPE_HINT_SPLASHSCREEN,
+  GDK_WINDOW_TYPE_HINT_UTILITY,
+  GDK_WINDOW_TYPE_HINT_DOCK,
+  GDK_WINDOW_TYPE_HINT_DESKTOP
 } GdkWindowTypeHint;
 
 
@@ -197,17 +201,13 @@ struct _GdkGeometry
   GdkGravity win_gravity;
 };
 
-/* Dummy typedef for placeholder in GdkPointerHooks.window_at_pointer
- */
-typedef struct _GdkScreen GdkScreen;
-
 struct _GdkPointerHooks 
 {
   GdkWindow* (*get_pointer)       (GdkWindow	   *window,
 			           gint	           *x,
 			           gint   	   *y,
 			           GdkModifierType *mask);
-  GdkWindow* (*window_at_pointer) (GdkScreen       *screen, /* unused for now*/
+  GdkWindow* (*window_at_pointer) (GdkScreen       *screen, /* unused */
                                    gint            *win_x,
                                    gint            *win_y);
 };
@@ -260,6 +260,8 @@ struct _GdkWindowObject
   guint modal_hint : 1;
   
   guint destroyed : 2;
+  
+  GdkEventMask event_mask;
 };
 
 struct _GdkWindowObjectClass
@@ -376,8 +378,14 @@ gboolean gdk_window_set_static_gravities (GdkWindow *window,
 					  gboolean   use_static);   
 
 /* Functions to create/lookup windows from their native equivalents */ 
+#ifndef GDK_MULTIHEAD_SAFE
 GdkWindow*    gdk_window_foreign_new (GdkNativeWindow anid);
 GdkWindow*    gdk_window_lookup      (GdkNativeWindow anid);
+#endif
+GdkWindow    *gdk_window_foreign_new_for_display (GdkDisplay      *display,
+						  GdkNativeWindow  anid);
+GdkWindow*    gdk_window_lookup_for_display (GdkDisplay      *display,
+					     GdkNativeWindow  anid);
 
 
 /* GdkWindow */
@@ -392,10 +400,16 @@ void	      gdk_window_set_hints	 (GdkWindow	  *window,
 					  gint		   max_height,
 					  gint		   flags);
 #endif
-void          gdk_window_set_type_hint   (GdkWindow       *window,
-					  GdkWindowTypeHint hint);
-void          gdk_window_set_modal_hint  (GdkWindow       *window,
-					  gboolean         modal);
+void          gdk_window_set_type_hint    (GdkWindow       *window,
+					   GdkWindowTypeHint hint);
+void          gdk_window_set_modal_hint   (GdkWindow       *window,
+                                           gboolean         modal);
+
+void gdk_window_set_skip_taskbar_hint (GdkWindow *window,
+                                       gboolean   skips_taskbar);
+void gdk_window_set_skip_pager_hint   (GdkWindow *window,
+                                       gboolean   skips_pager);
+
 void          gdk_window_set_geometry_hints (GdkWindow        *window,
 					     GdkGeometry      *geometry,
 					     GdkWindowHints    geom_mask);
@@ -435,7 +449,8 @@ gint	      gdk_window_get_origin	 (GdkWindow	  *window,
 					  gint		  *x,
 					  gint		  *y);
 
-#ifndef GDK_DISABLE_DEPRECATED
+#if !defined (GDK_DISABLE_DEPRECATED) || defined (GTK_COMPILATION)
+/* Used by gtk_handle_box_button_changed () */
 gboolean      gdk_window_get_deskrelative_origin (GdkWindow	  *window,
 					  gint		  *x,
 					  gint		  *y);
@@ -475,7 +490,9 @@ gboolean      gdk_window_get_decorations (GdkWindow       *window,
 					  GdkWMDecoration *decorations);
 void	      gdk_window_set_functions	 (GdkWindow	  *window,
 					  GdkWMFunction	   functions);
+#ifndef GDK_MULTIHEAD_SAFE
 GList *       gdk_window_get_toplevels   (void);
+#endif
 
 void          gdk_window_iconify         (GdkWindow       *window);
 void          gdk_window_deiconify       (GdkWindow       *window);
@@ -483,6 +500,8 @@ void          gdk_window_stick           (GdkWindow       *window);
 void          gdk_window_unstick         (GdkWindow       *window);
 void          gdk_window_maximize        (GdkWindow       *window);
 void          gdk_window_unmaximize      (GdkWindow       *window);
+void          gdk_window_fullscreen      (GdkWindow       *window);
+void          gdk_window_unfullscreen    (GdkWindow       *window);
 
 void          gdk_window_register_dnd    (GdkWindow       *window);
 
@@ -533,7 +552,9 @@ void gdk_window_get_internal_paint_info (GdkWindow    *window,
 					 gint         *x_offset,
 					 gint         *y_offset);
 
+#ifndef GDK_MULTIHEAD_SAFE
 GdkPointerHooks *gdk_set_pointer_hooks (const GdkPointerHooks *new_hooks);   
+#endif /* GDK_MULTIHEAD_SAFE */
 
 GdkWindow *gdk_get_default_root_window (void);
 
