@@ -36,7 +36,15 @@
 /* Do use SetCapture, it works now. Thanks to jpe@archaeopteryx.com */
 #define USE_SETCAPTURE 1
 
-#include <stdio.h>
+#include "config.h"
+
+#include <glib/gprintf.h>
+
+#if defined (__GNUC__) && defined (HAVE_DIMM_H)
+/* The w32api imm.h clashes a bit with the IE5.5 dimm.h */
+# define IMEMENUITEMINFOA hidden_IMEMENUITEMINFOA
+# define IMEMENUITEMINFOW hidden_IMEMENUITEMINFOW
+#endif
 
 #include "gdk.h"
 #include "gdkprivate-win32.h"
@@ -50,12 +58,6 @@
 
 #include <objbase.h>
 
-#if defined (__GNUC__) && defined (HAVE_DIMM_H)
-/* The w32api imm.h clashes a bit with the IE5.5 dimm.h */
-# define IMEMENUITEMINFOA hidden_IMEMENUITEMINFOA
-# define IMEMENUITEMINFOW hidden_IMEMENUITEMINFOW
-#endif
-
 #include <imm.h>
 
 #if defined (__GNUC__) && defined (HAVE_DIMM_H)
@@ -318,7 +320,7 @@ _gdk_events_init (void)
       while (latin_locale == NULL && i < G_N_ELEMENTS (latin_languages))
 	{
 	  char id[9];
-	  sprintf (id, "%08x", MAKELANGID (latin_languages[i++], SUBLANG_DEFAULT));
+	  g_sprintf (id, "%08x", MAKELANGID (latin_languages[i++], SUBLANG_DEFAULT));
 	  latin_locale = LoadKeyboardLayout (id, KLF_NOTELLSHELL|KLF_SUBSTITUTE_OK);
 	}
     }
@@ -451,7 +453,7 @@ event_mask_string (GdkEventMask mask)
   *p = '\0';
 #define BIT(x) \
   if (mask & GDK_##x##_MASK) \
-    p += sprintf (p, "%s" #x, (p > bfr ? " " : ""))
+    p += g_sprintf (p, "%s" #x, (p > bfr ? " " : ""))
   BIT (EXPOSURE);
   BIT (POINTER_MOTION);
   BIT (POINTER_MOTION_HINT);
@@ -775,6 +777,16 @@ gdk_event_apply_filters (MSG      *msg,
   return GDK_FILTER_CONTINUE;
 }
 
+void 
+gdk_display_add_client_message_filter (GdkDisplay   *display,
+				       GdkAtom       message_type,
+				       GdkFilterFunc func,
+				       gpointer      data)
+{
+  /* XXX */
+  gdk_add_client_message_filter (message_type, func, data);
+}
+
 void 
 gdk_add_client_message_filter (GdkAtom       message_type,
 			       GdkFilterFunc func,
@@ -1613,14 +1625,14 @@ decode_key_lparam (LPARAM lParam)
   char *p = buf;
 
   if (HIWORD (lParam) & KF_UP)
-    p += sprintf (p, "KF_UP ");
+    p += g_sprintf (p, "KF_UP ");
   if (HIWORD (lParam) & KF_REPEAT)
-    p += sprintf (p, "KF_REPEAT ");
+    p += g_sprintf (p, "KF_REPEAT ");
   if (HIWORD (lParam) & KF_ALTDOWN)
-    p += sprintf (p, "KF_ALTDOWN ");
+    p += g_sprintf (p, "KF_ALTDOWN ");
   if (HIWORD (lParam) & KF_EXTENDED)
-    p += sprintf (p, "KF_EXTENDED ");
-  p += sprintf (p, "sc:%d rep:%d", LOBYTE (HIWORD (lParam)), LOWORD (lParam));
+    p += g_sprintf (p, "KF_EXTENDED ");
+  p += g_sprintf (p, "sc:%d rep:%d", LOBYTE (HIWORD (lParam)), LOWORD (lParam));
 
   return buf;
 }
@@ -2986,6 +2998,9 @@ gdk_event_translate (GdkDisplay *display,
 #endif
 								 )
 	{
+	  GdkWindowState withdrawn_bit =
+	    IsWindowVisible (msg->hwnd) ? GDK_WINDOW_STATE_WITHDRAWN : 0;
+
 	  if (!(private->event_mask & GDK_STRUCTURE_MASK))
 	    break;
 
@@ -3007,12 +3022,12 @@ gdk_event_translate (GdkDisplay *display,
 	    gdk_synthesize_window_state (window,
 					 GDK_WINDOW_STATE_ICONIFIED |
 					 GDK_WINDOW_STATE_MAXIMIZED |
-					 GDK_WINDOW_STATE_WITHDRAWN,
+					 withdrawn_bit,
 					 0);
 	  else if (msg->wParam == SIZE_MAXIMIZED)
 	    gdk_synthesize_window_state (window,
 					 GDK_WINDOW_STATE_ICONIFIED |
-					 GDK_WINDOW_STATE_WITHDRAWN,
+					 withdrawn_bit,
 					 GDK_WINDOW_STATE_MAXIMIZED);
 
 	  if (private->resize_count > 1)
