@@ -23,6 +23,8 @@
 
 #include <config.h>
 
+#include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 
 #include <glib.h>
