@@ -37,8 +37,10 @@
 #include <string.h>
 #include <stdlib.h>
 #include <time.h>
-#include <stdio.h>
+#include <glib/gprintf.h>
+
 #include "gtkcalendar.h"
+#include "gtkmarshalers.h"
 #include "gdk/gdkkeysyms.h"
 
 /***************************************************************************/
@@ -177,14 +179,12 @@ dates_difference(N_int year1, N_int mm1, N_int dd1,
 /* Color usage */
 #define HEADER_FG_COLOR(widget)		 (& (widget)->style->fg[GTK_WIDGET_STATE (widget)])
 #define HEADER_BG_COLOR(widget)		 (& (widget)->style->bg[GTK_WIDGET_STATE (widget)])
-#define DAY_NAME_COLOR(widget)		 (& (widget)->style->base[GTK_STATE_SELECTED])
+#define SELECTED_BG_COLOR(widget)	 (& (widget)->style->base[GTK_WIDGET_HAS_FOCUS (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE])
+#define SELECTED_FG_COLOR(widget)	 (& (widget)->style->text[GTK_WIDGET_HAS_FOCUS (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE])
 #define NORMAL_DAY_COLOR(widget)	 (& (widget)->style->fg[GTK_WIDGET_STATE (widget)])
-#define SELECTION_FOCUS_COLOR(widget)	 (& (widget)->style->fg[GTK_WIDGET_STATE (widget)])
-#define SELECTION_NO_FOCUS_COLOR(widget) (& (widget)->style->mid[GTK_WIDGET_STATE (widget)])
 #define PREV_MONTH_COLOR(widget)	 (& (widget)->style->mid[GTK_WIDGET_STATE (widget)])
 #define NEXT_MONTH_COLOR(widget)	 (& (widget)->style->mid[GTK_WIDGET_STATE (widget)])
 #define MARKED_COLOR(widget)		 (& (widget)->style->fg[GTK_WIDGET_STATE (widget)])
-#define FOREGROUND_COLOR(widget)	 (& (widget)->style->fg[GTK_WIDGET_STATE (widget)])
 #define BACKGROUND_COLOR(widget)	 (& (widget)->style->base[GTK_WIDGET_STATE (widget)])
 #define HIGHLIGHT_BACK_COLOR(widget)	 (& (widget)->style->mid[GTK_WIDGET_STATE (widget)])
 
@@ -290,6 +290,8 @@ static void gtk_calendar_paint_day_names    (GtkWidget *widget);
 static void gtk_calendar_paint_week_numbers (GtkWidget *widget);
 static void gtk_calendar_paint_main	    (GtkWidget *widget);
 
+static void gtk_calendar_select_and_focus_day (GtkCalendar *calendar,
+					       guint        day);
 
 static void gtk_calendar_paint_arrow	(GtkWidget    *widget,
 					 guint	       arrow);
@@ -307,27 +309,28 @@ static gint top_y_for_row		(GtkCalendar  *calendar,
 static char    *default_abbreviated_dayname[7];
 static char    *default_monthname[12];
 
-GtkType
+GType
 gtk_calendar_get_type (void)
 {
-  static GtkType calendar_type = 0;
+  static GType calendar_type = 0;
   
   if (!calendar_type)
     {
       static const GTypeInfo calendar_info =
       {
 	sizeof (GtkCalendarClass),
-	NULL,           /* base_init */
-	NULL,           /* base_finalize */
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
 	(GClassInitFunc) gtk_calendar_class_init,
-	NULL,           /* class_finalize */
-	NULL,           /* class_data */
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
 	sizeof (GtkCalendar),
-	16,             /* n_preallocs */
+	0,		/* n_preallocs */
 	(GInstanceInitFunc) gtk_calendar_init,
       };
 
-      calendar_type = g_type_register_static (GTK_TYPE_WIDGET, "GtkCalendar", &calendar_info, 0);
+      calendar_type = g_type_register_static (GTK_TYPE_WIDGET, "GtkCalendar",
+					      &calendar_info, 0);
     }
   
   return calendar_type;
@@ -337,14 +340,12 @@ static void
 gtk_calendar_class_init (GtkCalendarClass *class)
 {
   GObjectClass   *gobject_class;
-  GtkObjectClass *object_class;
   GtkWidgetClass *widget_class;
 
   gobject_class = (GObjectClass*)  class;
-  object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
   
-  parent_class = gtk_type_class (GTK_TYPE_WIDGET);
+  parent_class = g_type_class_peek_parent (class);
   
   gobject_class->finalize = gtk_calendar_finalize;
 
@@ -370,40 +371,61 @@ gtk_calendar_class_init (GtkCalendarClass *class)
   class->next_year = NULL;
 
   gtk_calendar_signals[MONTH_CHANGED_SIGNAL] =
-    gtk_signal_new ("month_changed",
-		    GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkCalendarClass, month_changed),
-		    gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
+    g_signal_new ("month_changed",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkCalendarClass, month_changed),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   gtk_calendar_signals[DAY_SELECTED_SIGNAL] =
-    gtk_signal_new ("day_selected",
-		    GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkCalendarClass, day_selected),
-		    gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
+    g_signal_new ("day_selected",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkCalendarClass, day_selected),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   gtk_calendar_signals[DAY_SELECTED_DOUBLE_CLICK_SIGNAL] =
-    gtk_signal_new ("day_selected_double_click",
-		    GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkCalendarClass, day_selected_double_click),
-		    gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
+    g_signal_new ("day_selected_double_click",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkCalendarClass, day_selected_double_click),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   gtk_calendar_signals[PREV_MONTH_SIGNAL] =
-    gtk_signal_new ("prev_month",
-		    GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkCalendarClass, prev_month),
-		    gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
+    g_signal_new ("prev_month",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkCalendarClass, prev_month),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   gtk_calendar_signals[NEXT_MONTH_SIGNAL] =
-    gtk_signal_new ("next_month",
-		    GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkCalendarClass, next_month),
-		    gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
+    g_signal_new ("next_month",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkCalendarClass, next_month),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   gtk_calendar_signals[PREV_YEAR_SIGNAL] =
-    gtk_signal_new ("prev_year",
-		    GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkCalendarClass, prev_year),
-		    gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
+    g_signal_new ("prev_year",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkCalendarClass, prev_year),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   gtk_calendar_signals[NEXT_YEAR_SIGNAL] =
-    gtk_signal_new ("next_year",
-		    GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkCalendarClass, next_year),
-		    gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
+    g_signal_new ("next_year",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkCalendarClass, next_year),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 }
 
 static void
@@ -420,7 +442,7 @@ gtk_calendar_init (GtkCalendar *calendar)
   widget = GTK_WIDGET (calendar);
   GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
   
-  calendar->private_data = (gpointer) g_malloc (sizeof (GtkCalendarPrivateData));
+  calendar->private_data = g_malloc (sizeof (GtkCalendarPrivateData));
   private_data = GTK_CALENDAR_PRIVATE_DATA (calendar);
 
   if (!default_abbreviated_dayname[0])
@@ -483,7 +505,7 @@ gtk_calendar_init (GtkCalendar *calendar)
 GtkWidget*
 gtk_calendar_new (void)
 {
-  return GTK_WIDGET (gtk_type_new (GTK_TYPE_CALENDAR));
+  return g_object_new (GTK_TYPE_CALENDAR, NULL);
 }
 
 /* column_from_x: returns the column 0-6 that the
@@ -645,10 +667,12 @@ gtk_calendar_set_month_prev (GtkCalendar *calendar)
   gtk_calendar_freeze (calendar);
   gtk_calendar_compute_days (calendar);
   
-  gtk_signal_emit (GTK_OBJECT (calendar),
-		   gtk_calendar_signals[PREV_MONTH_SIGNAL]);
-  gtk_signal_emit (GTK_OBJECT (calendar),
-		   gtk_calendar_signals[MONTH_CHANGED_SIGNAL]);
+  g_signal_emit (calendar,
+		 gtk_calendar_signals[PREV_MONTH_SIGNAL],
+		 0);
+  g_signal_emit (calendar,
+		 gtk_calendar_signals[MONTH_CHANGED_SIGNAL],
+		 0);
   
   if (month_len < calendar->selected_day)
     {
@@ -688,10 +712,12 @@ gtk_calendar_set_month_next (GtkCalendar *calendar)
   
   gtk_calendar_freeze (calendar);
   gtk_calendar_compute_days (calendar);
-  gtk_signal_emit (GTK_OBJECT (calendar),
-		   gtk_calendar_signals[NEXT_MONTH_SIGNAL]);
-  gtk_signal_emit (GTK_OBJECT (calendar),
-		   gtk_calendar_signals[MONTH_CHANGED_SIGNAL]);
+  g_signal_emit (calendar,
+		 gtk_calendar_signals[NEXT_MONTH_SIGNAL],
+		 0);
+  g_signal_emit (calendar,
+		 gtk_calendar_signals[MONTH_CHANGED_SIGNAL],
+		 0);
   
   month_len = month_length[leap (calendar->year)][calendar->month + 1];
   
@@ -717,10 +743,12 @@ gtk_calendar_set_year_prev (GtkCalendar *calendar)
   calendar->year--;
   gtk_calendar_freeze (calendar);
   gtk_calendar_compute_days (calendar);
-  gtk_signal_emit (GTK_OBJECT (calendar),
-		   gtk_calendar_signals[PREV_YEAR_SIGNAL]);
-  gtk_signal_emit (GTK_OBJECT (calendar),
-		   gtk_calendar_signals[MONTH_CHANGED_SIGNAL]);
+  g_signal_emit (calendar,
+		 gtk_calendar_signals[PREV_YEAR_SIGNAL],
+		 0);
+  g_signal_emit (calendar,
+		 gtk_calendar_signals[MONTH_CHANGED_SIGNAL],
+		 0);
   
   month_len = month_length[leap (calendar->year)][calendar->month + 1];
   
@@ -750,10 +778,12 @@ gtk_calendar_set_year_next (GtkCalendar *calendar)
   
   calendar->year++;
   gtk_calendar_compute_days (calendar);
-  gtk_signal_emit (GTK_OBJECT (calendar),
-		   gtk_calendar_signals[NEXT_YEAR_SIGNAL]);
-  gtk_signal_emit (GTK_OBJECT (calendar),
-		   gtk_calendar_signals[MONTH_CHANGED_SIGNAL]);
+  g_signal_emit (calendar,
+		 gtk_calendar_signals[NEXT_YEAR_SIGNAL],
+		 0);
+  g_signal_emit (calendar,
+		 gtk_calendar_signals[MONTH_CHANGED_SIGNAL],
+		 0);
   
   month_len = month_length[leap (calendar->year)][calendar->month + 1];
   
@@ -777,7 +807,7 @@ gtk_calendar_main_button (GtkWidget	 *widget,
   gint x, y;
   gint row, col;
   gint day_month;
-  gint old_focus_row, old_focus_col;
+  gint day;
   
   calendar = GTK_CALENDAR (widget);
   
@@ -792,28 +822,29 @@ gtk_calendar_main_button (GtkWidget	 *widget,
     return;
   
   day_month = calendar->day_month[row][col];
-  
-  if (day_month == MONTH_CURRENT)
+
+  if (event->type == GDK_BUTTON_PRESS)
     {
-      if (event->type == GDK_2BUTTON_PRESS)
-	gtk_signal_emit (GTK_OBJECT (calendar),
-			 gtk_calendar_signals[DAY_SELECTED_DOUBLE_CLICK_SIGNAL]);
-      else
-	{
-	  if (!GTK_WIDGET_HAS_FOCUS (widget))
-	    gtk_widget_grab_focus (widget);
-	  old_focus_row = calendar->focus_row;
-	  old_focus_col = calendar->focus_col;
-	  calendar->focus_row = row;
-	  calendar->focus_col = col;
-	  gtk_calendar_paint_day (widget, old_focus_row, old_focus_col);
-	  gtk_calendar_select_day (calendar, calendar->day[row][col]);
-	}
+      day = calendar->day[row][col];
+      
+      if (day_month == MONTH_PREV)
+	gtk_calendar_set_month_prev (calendar);
+      else if (day_month == MONTH_NEXT)
+	gtk_calendar_set_month_next (calendar);
+      
+      if (!GTK_WIDGET_HAS_FOCUS (widget))
+	gtk_widget_grab_focus (widget);
+	  
+      gtk_calendar_select_and_focus_day (calendar, day);
+
+    }
+  else if (event->type == GDK_2BUTTON_PRESS)
+    {
+      if (day_month == MONTH_CURRENT)
+	g_signal_emit (calendar,
+		       gtk_calendar_signals[DAY_SELECTED_DOUBLE_CLICK_SIGNAL],
+		       0);
     }
-  else if (day_month == MONTH_PREV)
-    gtk_calendar_set_month_prev (calendar);
-  else if (day_month == MONTH_NEXT)
-    gtk_calendar_set_month_next (calendar);
 }
 
 static void
@@ -1125,9 +1156,9 @@ gtk_calendar_unrealize (GtkWidget *widget)
       private_data->day_name_win = NULL;      
     }
   if (calendar->xor_gc)
-    gdk_gc_unref (calendar->xor_gc);
+    g_object_unref (calendar->xor_gc);
   if (calendar->gc)
-    gdk_gc_unref (calendar->gc);
+    g_object_unref (calendar->gc);
   
   if (GTK_WIDGET_CLASS (parent_class)->unrealize)
     (* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
@@ -1180,7 +1211,7 @@ gtk_calendar_size_request (GtkWidget	  *widget,
       private_data->max_year_width = 0;
       for (i=0; i<10; i++)
 	{
-	  sprintf (buffer, "%d%d%d%d", i,i,i,i);
+	  g_snprintf (buffer, sizeof (buffer), "%d%d%d%d", i,i,i,i);
 	  pango_layout_set_text (layout, buffer, -1);	  
 	  pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 	  private_data->max_year_width = MAX (private_data->max_year_width,
@@ -1212,7 +1243,7 @@ gtk_calendar_size_request (GtkWidget	  *widget,
 
   for (i = 0; i < 9; i++)
     {
-      sprintf (buffer, "%d%d", i, i);
+      g_snprintf (buffer, sizeof (buffer), "%d%d", i, i);
       pango_layout_set_text (layout, buffer, -1);	  
       pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
       private_data->min_day_width = MAX (private_data->min_day_width,
@@ -1243,7 +1274,7 @@ gtk_calendar_size_request (GtkWidget	  *widget,
   if (calendar->display_flags & GTK_CALENDAR_SHOW_WEEK_NUMBERS)
     for (i = 0; i < 9; i++)
       {
-	sprintf (buffer, "%d%d", i, i);
+	g_snprintf (buffer, sizeof (buffer), "%d%d", i, i);
 	pango_layout_set_text (layout, buffer, -1);	  
 	pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 	private_data->max_week_char_width = MAX (private_data->max_week_char_width,
@@ -1294,7 +1325,7 @@ gtk_calendar_size_request (GtkWidget	  *widget,
   
   requisition->height = height + (widget->style->ythickness + INNER_BORDER) * 2;
 
-  g_object_unref (G_OBJECT (layout));
+  g_object_unref (layout);
 }
 
 static void
@@ -1459,12 +1490,13 @@ gtk_calendar_paint_header (GtkWidget *widget)
   max_year_width = private_data->max_year_width;
   
   gdk_gc_set_foreground (gc, BACKGROUND_COLOR (GTK_WIDGET (calendar)));
-  gtk_draw_shadow (widget->style, private_data->header_win,
-		   GTK_STATE_NORMAL, GTK_SHADOW_OUT,
-		   0, 0, header_width, private_data->header_h);
+  gtk_paint_shadow (widget->style, private_data->header_win,
+		    GTK_STATE_NORMAL, GTK_SHADOW_OUT,
+		    NULL, widget, "calendar",
+		    0, 0, header_width, private_data->header_h);
   
   
-  sprintf (buffer, "%d", calendar->year);
+  g_snprintf (buffer, sizeof (buffer), "%d", calendar->year);
   layout = gtk_widget_create_pango_layout (widget, buffer);
   pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
   
@@ -1485,7 +1517,7 @@ gtk_calendar_paint_header (GtkWidget *widget)
   gdk_draw_layout (private_data->header_win, gc, x, y, layout);
   
   /* Draw month */
-  sprintf (buffer, "%s", default_monthname[calendar->month]);
+  g_snprintf (buffer, sizeof (buffer), "%s", default_monthname[calendar->month]);
   pango_layout_set_text (layout, buffer, -1);
   pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 
@@ -1503,7 +1535,7 @@ gtk_calendar_paint_header (GtkWidget *widget)
   gtk_calendar_paint_arrow (widget, ARROW_YEAR_LEFT);
   gtk_calendar_paint_arrow (widget, ARROW_YEAR_RIGHT);
 
-  g_object_unref (G_OBJECT (layout));
+  g_object_unref (layout);
 }
 
 static void
@@ -1557,7 +1589,7 @@ gtk_calendar_paint_day_names (GtkWidget *widget)
    * Draw rectangles as inverted background for the labels.
    */
   
-  gdk_gc_set_foreground (gc, &widget->style->base[GTK_STATE_SELECTED]);
+  gdk_gc_set_foreground (gc, SELECTED_BG_COLOR (widget));
   gdk_draw_rectangle (private_data->day_name_win, gc, TRUE,
 		      CALENDAR_MARGIN, CALENDAR_MARGIN,
 		      cal_width-CALENDAR_MARGIN * 2,
@@ -1576,7 +1608,7 @@ gtk_calendar_paint_day_names (GtkWidget *widget)
 
   layout = gtk_widget_create_pango_layout (widget, NULL);
   
-  gdk_gc_set_foreground (gc, &widget->style->text[GTK_STATE_SELECTED]);
+  gdk_gc_set_foreground (gc, SELECTED_FG_COLOR (widget));
   for (i = 0; i < 7; i++)
     {
       if (gtk_widget_get_direction (GTK_WIDGET (calendar)) == GTK_TEXT_DIR_RTL)
@@ -1585,7 +1617,7 @@ gtk_calendar_paint_day_names (GtkWidget *widget)
 	day = i;
       if (calendar->display_flags & GTK_CALENDAR_WEEK_START_MONDAY)
 	day= (day+1)%7;
-      sprintf (buffer, "%s", default_abbreviated_dayname[day]);
+      g_snprintf (buffer, sizeof (buffer), "%s", default_abbreviated_dayname[day]);
 
       pango_layout_set_text (layout, buffer, -1);
       pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
@@ -1600,7 +1632,7 @@ gtk_calendar_paint_day_names (GtkWidget *widget)
 		       layout);
     }
   
-  g_object_unref (G_OBJECT (layout));
+  g_object_unref (layout);
 }
 
 static void
@@ -1650,7 +1682,7 @@ gtk_calendar_paint_week_numbers (GtkWidget *widget)
    * Draw a rectangle as inverted background for the labels.
    */
   
-  gdk_gc_set_foreground (gc, &widget->style->base[GTK_STATE_SELECTED]);
+  gdk_gc_set_foreground (gc, SELECTED_BG_COLOR (widget));
   if (private_data->day_name_win)
     gdk_draw_rectangle (private_data->week_win, gc, TRUE,
 			CALENDAR_MARGIN,
@@ -1670,7 +1702,7 @@ gtk_calendar_paint_week_numbers (GtkWidget *widget)
   
   layout = gtk_widget_create_pango_layout (widget, NULL);
   
-  gdk_gc_set_foreground (gc, &widget->style->text[GTK_STATE_SELECTED]);
+  gdk_gc_set_foreground (gc, SELECTED_FG_COLOR (widget));
   day_height = row_height (calendar);
   for (row = 0; row < 6; row++)
     {
@@ -1682,7 +1714,7 @@ gtk_calendar_paint_week_numbers (GtkWidget *widget)
 				      ((calendar->day[row][6] < 15 && row > 3 ? 1 : 0)
 				       + calendar->month) % 12 + 1, calendar->day[row][6]));
 
-      sprintf (buffer, "%d", week);
+      g_snprintf (buffer, sizeof (buffer), "%d", week);
       pango_layout_set_text (layout, buffer, -1);
       pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 
@@ -1695,7 +1727,7 @@ gtk_calendar_paint_week_numbers (GtkWidget *widget)
       gdk_draw_layout (private_data->week_win, gc, x_loc, y_loc, layout);
     }
   
-  g_object_unref (G_OBJECT (layout));
+  g_object_unref (layout);
 }
 
 static void
@@ -1801,7 +1833,7 @@ gtk_calendar_paint_day (GtkWidget *widget,
       */
       if (calendar->selected_day == day)
 	{
-	  gdk_gc_set_foreground (gc, & (GTK_WIDGET (calendar)->style->base[GTK_STATE_SELECTED]));
+	  gdk_gc_set_foreground (gc, SELECTED_BG_COLOR (GTK_WIDGET (calendar)));
 	  gdk_draw_rectangle (private_data->main_win, gc, TRUE, x_left, y_top,
 			      private_data->day_width, day_height);
 	}
@@ -1812,13 +1844,13 @@ gtk_calendar_paint_day (GtkWidget *widget,
 	gdk_gc_set_foreground (gc, NORMAL_DAY_COLOR (GTK_WIDGET (calendar)));
   
       if (calendar->selected_day == day)
-	gdk_gc_set_foreground (gc, & (GTK_WIDGET (calendar)->style->text[GTK_STATE_SELECTED]));
+	gdk_gc_set_foreground (gc, SELECTED_FG_COLOR (GTK_WIDGET (calendar)));
       else
 	gdk_gc_set_foreground (gc, & (GTK_WIDGET (calendar)->style->fg[GTK_WIDGET_STATE (calendar)]));
     }
     
 
-  sprintf (buffer, "%d", day);
+  g_snprintf (buffer, sizeof (buffer), "%d", day);
   layout = gtk_widget_create_pango_layout (widget, buffer);
   pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
   
@@ -1835,6 +1867,13 @@ gtk_calendar_paint_day (GtkWidget *widget,
   if (GTK_WIDGET_HAS_FOCUS (calendar) 
       && calendar->focus_row == row && calendar->focus_col == col)
     {
+      GtkStateType state;
+
+      if (calendar->selected_day == day)
+	state = GTK_WIDGET_HAS_FOCUS (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
+      else
+	state = GTK_STATE_NORMAL;
+      
       gtk_paint_focus (widget->style, 
 		       private_data->main_win,
 		       (calendar->selected_day == day) 
@@ -1845,7 +1884,7 @@ gtk_calendar_paint_day (GtkWidget *widget,
 		       day_height);
     }
 
-  g_object_unref (G_OBJECT (layout));
+  g_object_unref (layout);
 }
 
 
@@ -2103,8 +2142,9 @@ gtk_calendar_select_month (GtkCalendar *calendar,
   gtk_calendar_compute_days (calendar);
   
   gtk_widget_queue_draw (GTK_WIDGET (calendar));
-  gtk_signal_emit (GTK_OBJECT (calendar),
-		   gtk_calendar_signals[MONTH_CHANGED_SIGNAL]);
+  g_signal_emit (calendar,
+		 gtk_calendar_signals[MONTH_CHANGED_SIGNAL],
+		 0);
   return TRUE;
 }
 
@@ -2130,15 +2170,42 @@ gtk_calendar_select_day (GtkCalendar *calendar,
   
   calendar->selected_day = day;
   
-  /* Deselect the new day */
+  /* Select the new day */
   if (day != 0)
     {
       if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (calendar)))
 	gtk_calendar_paint_day_num (GTK_WIDGET (calendar), day);
     }
   
-  gtk_signal_emit (GTK_OBJECT (calendar),
-		   gtk_calendar_signals[DAY_SELECTED_SIGNAL]);
+  g_signal_emit (calendar,
+		 gtk_calendar_signals[DAY_SELECTED_SIGNAL],
+		 0);
+}
+
+static void
+gtk_calendar_select_and_focus_day (GtkCalendar *calendar,
+				   guint        day)
+{
+  gint old_focus_row = calendar->focus_row;
+  gint old_focus_col = calendar->focus_col;
+  gint row;
+  gint col;
+  
+  for (row = 0; row < 6; row ++)
+    for (col = 0; col < 7; col++)
+      {
+	if (calendar->day_month[row][col] == MONTH_CURRENT 
+	    && calendar->day[row][col] == day)
+	  {
+	    calendar->focus_row = row;
+	    calendar->focus_col = col;
+	  }
+      }
+
+  if (old_focus_row != -1 && old_focus_col != -1)
+    gtk_calendar_paint_day (GTK_WIDGET (calendar), old_focus_row, old_focus_col);
+  
+  gtk_calendar_select_day (calendar, day);
 }
 
 void
@@ -2409,7 +2476,7 @@ gtk_calendar_paint_arrow (GtkWidget *widget,
       
       gdk_window_clear (window);
       gdk_window_set_background (window, &(widget)->style->bg[state]);
-      gdk_window_get_size (window, &width, &height);
+      gdk_drawable_get_size (window, &width, &height);
       gdk_window_clear_area (window,
 			     0,0,
 			     width,height);
@@ -2645,18 +2712,8 @@ gtk_calendar_key_press (GtkWidget   *widget,
 	      gtk_calendar_set_month_next (calendar);
 	    }
 
-	  gtk_calendar_select_day (calendar, day);
+	  gtk_calendar_select_and_focus_day (calendar, day);
 	  
-	  for (row = 0; row < 6; row ++)
-	    for (col = 0; col < 7; col++)
-	      {
-		if (calendar->day_month[row][col] == MONTH_CURRENT 
-		    && calendar->day[row][col] == day)
-		  {
-		    calendar->focus_row = row;
-		    calendar->focus_col = col;
-		  }
-	      }
 	  gtk_calendar_thaw (calendar);	  
 	}
     }	
