@@ -29,16 +29,24 @@
 #include "gtkhandlebox.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
-#include "gtksignal.h"
 #include "gtkwindow.h"
 #include "gtkintl.h"
 
+typedef struct _GtkHandleBoxPrivate GtkHandleBoxPrivate;
+
+struct _GtkHandleBoxPrivate
+{
+  gint orig_x;
+  gint orig_y;
+};
+
 enum {
   PROP_0,
   PROP_SHADOW,
   PROP_SHADOW_TYPE,
   PROP_HANDLE_POSITION,
-  PROP_SNAP_EDGE
+  PROP_SNAP_EDGE,
+  PROP_SNAP_EDGE_SET
 };
 
 #define DRAG_HANDLE_SIZE 10
@@ -136,26 +144,28 @@ static GtkBinClass *parent_class;
 static guint        handle_box_signals[SIGNAL_LAST] = { 0 };
 
 
-GtkType
+GType
 gtk_handle_box_get_type (void)
 {
-  static GtkType handle_box_type = 0;
+  static GType handle_box_type = 0;
 
   if (!handle_box_type)
     {
-      static const GtkTypeInfo handle_box_info =
+      static const GTypeInfo handle_box_info =
       {
-	"GtkHandleBox",
-	sizeof (GtkHandleBox),
 	sizeof (GtkHandleBoxClass),
-	(GtkClassInitFunc) gtk_handle_box_class_init,
-	(GtkObjectInitFunc) gtk_handle_box_init,
-	/* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_handle_box_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkHandleBox),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_handle_box_init,
       };
 
-      handle_box_type = gtk_type_unique (GTK_TYPE_BIN, &handle_box_info);
+      handle_box_type = g_type_register_static (GTK_TYPE_BIN, "GtkHandleBox",
+						&handle_box_info, 0);
     }
 
   return handle_box_type;
@@ -174,7 +184,7 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
   widget_class = (GtkWidgetClass *) class;
   container_class = (GtkContainerClass *) class;
 
-  parent_class = gtk_type_class (GTK_TYPE_BIN);
+  parent_class = g_type_class_peek_parent (class);
 
   gobject_class->set_property = gtk_handle_box_set_property;
   gobject_class->get_property = gtk_handle_box_get_property;
@@ -212,6 +222,15 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
 						      GTK_TYPE_POSITION_TYPE,
 						      GTK_POS_TOP,
                                                       G_PARAM_READABLE | G_PARAM_WRITABLE));
+
+  g_object_class_install_property (gobject_class,
+                                   PROP_SNAP_EDGE_SET,
+                                   g_param_spec_boolean ("snap_edge_set",
+							 _("Snap edge set"),
+							 _("Whether to use the value from the snap_edge property or a value derived from handle_position"),
+							 FALSE,
+							 G_PARAM_READABLE | G_PARAM_WRITABLE));
+
   object_class->destroy = gtk_handle_box_destroy;
 
   widget_class->map = gtk_handle_box_map;
@@ -234,21 +253,44 @@ gtk_handle_box_class_init (GtkHandleBoxClass *class)
   class->child_detached = NULL;
 
   handle_box_signals[SIGNAL_CHILD_ATTACHED] =
-    gtk_signal_new ("child_attached",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkHandleBoxClass, child_attached),
-		    _gtk_marshal_VOID__OBJECT,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_WIDGET);
+    g_signal_new ("child_attached",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkHandleBoxClass, child_attached),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  GTK_TYPE_WIDGET);
   handle_box_signals[SIGNAL_CHILD_DETACHED] =
-    gtk_signal_new ("child_detached",
-		    GTK_RUN_FIRST,
-		    GTK_CLASS_TYPE (object_class),
-		    GTK_SIGNAL_OFFSET (GtkHandleBoxClass, child_detached),
-		    _gtk_marshal_VOID__OBJECT,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_WIDGET);
+    g_signal_new ("child_detached",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkHandleBoxClass, child_detached),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  GTK_TYPE_WIDGET);
+}
+
+GtkHandleBoxPrivate *
+gtk_handle_box_get_private (GtkHandleBox *hb)
+{
+  GtkHandleBoxPrivate *private;
+  static GQuark private_quark = 0;
+
+  if (!private_quark)
+    private_quark = g_quark_from_static_string ("gtk-handle-box-private");
+
+  private = g_object_get_qdata (G_OBJECT (hb), private_quark);
+
+  if (!private)
+    {
+      private = g_new0 (GtkHandleBoxPrivate, 1);
+      g_object_set_qdata_full (G_OBJECT (hb), private_quark,
+			       private, g_free);
+    }
+
+  return private;
 }
 
 static void
@@ -287,6 +329,10 @@ gtk_handle_box_set_property (GObject         *object,
     case PROP_SNAP_EDGE:
       gtk_handle_box_set_snap_edge (handle_box, g_value_get_enum (value));
       break;
+    case PROP_SNAP_EDGE_SET:
+      if (!g_value_get_boolean (value))
+	gtk_handle_box_set_snap_edge (handle_box, (GtkPositionType)-1);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -311,7 +357,12 @@ gtk_handle_box_get_property (GObject         *object,
       g_value_set_enum (value, handle_box->handle_position);
       break;
     case PROP_SNAP_EDGE:
-      g_value_set_enum (value, handle_box->snap_edge);
+      g_value_set_enum (value,
+			(handle_box->snap_edge == -1 ?
+			 GTK_POS_TOP : handle_box->snap_edge));
+      break;
+    case PROP_SNAP_EDGE_SET:
+      g_value_set_boolean (value, handle_box->snap_edge != -1);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -322,7 +373,7 @@ gtk_handle_box_get_property (GObject         *object,
 GtkWidget*
 gtk_handle_box_new (void)
 {
-  return GTK_WIDGET (gtk_type_new (gtk_handle_box_get_type ()));
+  return g_object_new (GTK_TYPE_HANDLE_BOX, NULL);
 }
 
 static void
@@ -477,6 +528,32 @@ gtk_handle_box_style_set (GtkWidget *widget,
     }
 }
 
+static int
+effective_handle_position (GtkHandleBox *hb)
+{
+  int handle_position;
+
+  if (gtk_widget_get_direction (GTK_WIDGET (hb)) == GTK_TEXT_DIR_LTR)
+    handle_position = hb->handle_position;
+  else
+    {
+      switch (hb->handle_position) 
+	{
+	case GTK_POS_LEFT:
+	  handle_position = GTK_POS_RIGHT;
+	  break;
+	case GTK_POS_RIGHT:
+	  handle_position = GTK_POS_LEFT;
+	  break;
+	default:
+	  handle_position = hb->handle_position;
+	  break;
+	}
+    }
+
+  return handle_position;
+}
+
 static void
 gtk_handle_box_size_request (GtkWidget      *widget,
 			     GtkRequisition *requisition)
@@ -484,12 +561,15 @@ gtk_handle_box_size_request (GtkWidget      *widget,
   GtkBin *bin;
   GtkHandleBox *hb;
   GtkRequisition child_requisition;
+  gint handle_position;
 
   bin = GTK_BIN (widget);
   hb = GTK_HANDLE_BOX (widget);
 
-  if (hb->handle_position == GTK_POS_LEFT ||
-      hb->handle_position == GTK_POS_RIGHT)
+  handle_position = effective_handle_position (hb);
+
+  if (handle_position == GTK_POS_LEFT ||
+      handle_position == GTK_POS_RIGHT)
     {
       requisition->width = DRAG_HANDLE_SIZE;
       requisition->height = 0;
@@ -516,16 +596,16 @@ gtk_handle_box_size_request (GtkWidget      *widget,
       /* FIXME: This doesn't work currently */
       if (!hb->shrink_on_detach)
 	{
-	  if (hb->handle_position == GTK_POS_LEFT ||
-	      hb->handle_position == GTK_POS_RIGHT)
+	  if (handle_position == GTK_POS_LEFT ||
+	      handle_position == GTK_POS_RIGHT)
 	    requisition->height += child_requisition.height;
 	  else
 	    requisition->width += child_requisition.width;
 	}
       else
 	{
-	  if (hb->handle_position == GTK_POS_LEFT ||
-	      hb->handle_position == GTK_POS_RIGHT)
+	  if (handle_position == GTK_POS_LEFT ||
+	      handle_position == GTK_POS_RIGHT)
 	    requisition->height += widget->style->ythickness;
 	  else
 	    requisition->width += widget->style->xthickness;
@@ -556,10 +636,13 @@ gtk_handle_box_size_allocate (GtkWidget     *widget,
   GtkBin *bin;
   GtkHandleBox *hb;
   GtkRequisition child_requisition;
+  gint handle_position;
   
   bin = GTK_BIN (widget);
   hb = GTK_HANDLE_BOX (widget);
   
+  handle_position = effective_handle_position (hb);
+
   if (bin->child)
     gtk_widget_get_child_requisition (bin->child, &child_requisition);
   else
@@ -589,9 +672,9 @@ gtk_handle_box_size_allocate (GtkWidget     *widget,
 
       child_allocation.x = border_width;
       child_allocation.y = border_width;
-      if (hb->handle_position == GTK_POS_LEFT)
+      if (handle_position == GTK_POS_LEFT)
 	child_allocation.x += DRAG_HANDLE_SIZE;
-      else if (hb->handle_position == GTK_POS_TOP)
+      else if (handle_position == GTK_POS_TOP)
 	child_allocation.y += DRAG_HANDLE_SIZE;
 
       if (hb->child_detached)
@@ -605,8 +688,8 @@ gtk_handle_box_size_allocate (GtkWidget     *widget,
 	  float_width = child_allocation.width + 2 * border_width;
 	  float_height = child_allocation.height + 2 * border_width;
 	  
-	  if (hb->handle_position == GTK_POS_LEFT ||
-	      hb->handle_position == GTK_POS_RIGHT)
+	  if (handle_position == GTK_POS_LEFT ||
+	      handle_position == GTK_POS_RIGHT)
 	    float_width += DRAG_HANDLE_SIZE;
 	  else
 	    float_height += DRAG_HANDLE_SIZE;
@@ -628,8 +711,8 @@ gtk_handle_box_size_allocate (GtkWidget     *widget,
 	  child_allocation.width = MAX (1, (gint)widget->allocation.width - 2 * border_width);
 	  child_allocation.height = MAX (1, (gint)widget->allocation.height - 2 * border_width);
 
-	  if (hb->handle_position == GTK_POS_LEFT ||
-	      hb->handle_position == GTK_POS_RIGHT)
+	  if (handle_position == GTK_POS_LEFT ||
+	      handle_position == GTK_POS_RIGHT)
 	    child_allocation.width -= DRAG_HANDLE_SIZE;
 	  else
 	    child_allocation.height -= DRAG_HANDLE_SIZE;
@@ -654,13 +737,15 @@ gtk_handle_box_draw_ghost (GtkHandleBox *hb)
   guint y;
   guint width;
   guint height;
+  gint handle_position;
 
   widget = GTK_WIDGET (hb);
-
-  if (hb->handle_position == GTK_POS_LEFT ||
-      hb->handle_position == GTK_POS_RIGHT)
+  
+  handle_position = effective_handle_position (hb);
+  if (handle_position == GTK_POS_LEFT ||
+      handle_position == GTK_POS_RIGHT)
     {
-      x = hb->handle_position == GTK_POS_LEFT ? 0 : widget->allocation.width - DRAG_HANDLE_SIZE;
+      x = handle_position == GTK_POS_LEFT ? 0 : widget->allocation.width - DRAG_HANDLE_SIZE;
       y = 0;
       width = DRAG_HANDLE_SIZE;
       height = widget->allocation.height;
@@ -668,7 +753,7 @@ gtk_handle_box_draw_ghost (GtkHandleBox *hb)
   else
     {
       x = 0;
-      y = hb->handle_position == GTK_POS_TOP ? 0 : widget->allocation.height - DRAG_HANDLE_SIZE;
+      y = handle_position == GTK_POS_TOP ? 0 : widget->allocation.height - DRAG_HANDLE_SIZE;
       width = widget->allocation.width;
       height = DRAG_HANDLE_SIZE;
     }
@@ -681,22 +766,22 @@ gtk_handle_box_draw_ghost (GtkHandleBox *hb)
 		    y,
 		    width,
 		    height);
-   if (hb->handle_position == GTK_POS_LEFT ||
-       hb->handle_position == GTK_POS_RIGHT)
+   if (handle_position == GTK_POS_LEFT ||
+       handle_position == GTK_POS_RIGHT)
      gtk_paint_hline (widget->style,
 		      widget->window,
 		      GTK_WIDGET_STATE (widget),
 		      NULL, widget, "handlebox",
-		      hb->handle_position == GTK_POS_LEFT ? DRAG_HANDLE_SIZE : 0,
-		      hb->handle_position == GTK_POS_LEFT ? widget->allocation.width : widget->allocation.width - DRAG_HANDLE_SIZE,
+		      handle_position == GTK_POS_LEFT ? DRAG_HANDLE_SIZE : 0,
+		      handle_position == GTK_POS_LEFT ? widget->allocation.width : widget->allocation.width - DRAG_HANDLE_SIZE,
 		      widget->allocation.height / 2);
    else
      gtk_paint_vline (widget->style,
 		      widget->window,
 		      GTK_WIDGET_STATE (widget),
 		      NULL, widget, "handlebox",
-		      hb->handle_position == GTK_POS_TOP ? DRAG_HANDLE_SIZE : 0,
-		      hb->handle_position == GTK_POS_TOP ? widget->allocation.height : widget->allocation.height - DRAG_HANDLE_SIZE,
+		      handle_position == GTK_POS_TOP ? DRAG_HANDLE_SIZE : 0,
+		      handle_position == GTK_POS_TOP ? widget->allocation.height : widget->allocation.height - DRAG_HANDLE_SIZE,
 		      widget->allocation.width / 2);
 }
 
@@ -781,7 +866,11 @@ gtk_handle_box_set_snap_edge        (GtkHandleBox    *handle_box,
   if (handle_box->snap_edge != edge)
     {
       handle_box->snap_edge = edge;
+      
+      g_object_freeze_notify (G_OBJECT (handle_box));
       g_object_notify (G_OBJECT (handle_box), "snap_edge");
+      g_object_notify (G_OBJECT (handle_box), "snap_edge_set");
+      g_object_thaw_notify (G_OBJECT (handle_box));
     }
 }
 
@@ -805,7 +894,8 @@ gtk_handle_box_get_snap_edge (GtkHandleBox *handle_box)
 
 static void
 gtk_handle_box_paint (GtkWidget      *widget,
-		      GdkEventExpose *event,
+		  
+    GdkEventExpose *event,
 		      GdkRectangle   *area)
 {
   GtkBin *bin;
@@ -814,11 +904,14 @@ gtk_handle_box_paint (GtkWidget      *widget,
   guint height;
   GdkRectangle rect;
   GdkRectangle dest;
+  gint handle_position;
 
   bin = GTK_BIN (widget);
   hb = GTK_HANDLE_BOX (widget);
 
-  gdk_window_get_size (hb->bin_window, &width, &height);
+  handle_position = effective_handle_position (hb);
+
+  gdk_drawable_get_size (hb->bin_window, &width, &height);
   
   if (!event)
     gtk_paint_box (widget->style,
@@ -843,7 +936,7 @@ gtk_handle_box_paint (GtkWidget      *widget,
 		 width,
 		 height);*/
 
-  switch (hb->handle_position)
+  switch (handle_position)
     {
     case GTK_POS_LEFT:
       rect.x = 0;
@@ -877,17 +970,7 @@ gtk_handle_box_paint (GtkWidget      *widget,
 			 event ? &event->area : area);
 
   if (bin->child && GTK_WIDGET_VISIBLE (bin->child))
-    {
-      GdkRectangle child_area;
-
-      if (!event) /* we were called from draw() */
-	{
-	  if (gtk_widget_intersect (bin->child, area, &child_area))
-	    gtk_widget_draw (bin->child, &child_area);
-	}
-      else /* we were called from expose() */
-	(* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
-    }
+    (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
 }
 
 static gint
@@ -919,9 +1002,12 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
   GtkHandleBox *hb;
   gboolean event_handled;
   GdkCursor *fleur;
+  gint handle_position;
 
   hb = GTK_HANDLE_BOX (widget);
 
+  handle_position = effective_handle_position (hb);
+
   event_handled = FALSE;
   if ((event->button == 1) && 
       (event->type == GDK_BUTTON_PRESS || event->type == GDK_2BUTTON_PRESS))
@@ -936,7 +1022,7 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
 
       if (child)
 	{
-	  switch (hb->handle_position)
+	  switch (handle_position)
 	    {
 	    case GTK_POS_LEFT:
 	      in_handle = event->x < DRAG_HANDLE_SIZE;
@@ -965,13 +1051,17 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
 	{
 	  if (event->type == GDK_BUTTON_PRESS) /* Start a drag */
 	    {
+	      GtkHandleBoxPrivate *private = gtk_handle_box_get_private (hb);
 	      gint desk_x, desk_y;
 	      gint root_x, root_y;
 	      gint width, height;
-	      
+
 	      gdk_window_get_deskrelative_origin (hb->bin_window, &desk_x, &desk_y);
 	      gdk_window_get_origin (hb->bin_window, &root_x, &root_y);
-	      gdk_window_get_size (hb->bin_window, &width, &height);
+	      gdk_drawable_get_size (hb->bin_window, &width, &height);
+
+	      private->orig_x = event->x_root;
+	      private->orig_y = event->y_root;
 	      
 	      hb->float_allocation.x = root_x - event->x_root;
 	      hb->float_allocation.y = root_y - event->y_root;
@@ -982,7 +1072,7 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
 	      hb->deskoff_y = desk_y - root_y;
 	      
 	      gdk_window_get_origin (widget->window, &root_x, &root_y);
-	      gdk_window_get_size (widget->window, &width, &height);
+	      gdk_drawable_get_size (widget->window, &width, &height);
 	      
 	      hb->attach_allocation.x = root_x;
 	      hb->attach_allocation.y = root_y;
@@ -1004,7 +1094,7 @@ gtk_handle_box_button_changed (GtkWidget      *widget,
 		  hb->in_drag = FALSE;
 		}
 	      
-	      gdk_cursor_destroy (fleur);
+	      gdk_cursor_unref (fleur);
 	      event_handled = TRUE;
 	    }
 	  else if (hb->child_detached) /* Double click */
@@ -1036,6 +1126,9 @@ gtk_handle_box_motion (GtkWidget      *widget,
   gint new_x, new_y;
   gint snap_edge;
   gboolean is_snapped = FALSE;
+  gint handle_position;
+  GdkGeometry geometry;
+  GdkScreen *screen, *pointer_screen;
 
   hb = GTK_HANDLE_BOX (widget);
   if (!hb->in_drag)
@@ -1044,22 +1137,47 @@ gtk_handle_box_motion (GtkWidget      *widget,
   if (!hb->in_drag || (event->window != widget->window))
     return FALSE;
   
+  handle_position = effective_handle_position (hb);
+
   /* Calculate the attachment point on the float, if the float
    * were detached
    */
   new_x = 0;
   new_y = 0;
-  gdk_window_get_pointer (gtk_widget_get_root_window (widget), 
-			  &new_x, &new_y, NULL);
+  screen = gtk_widget_get_screen (widget);
+  gdk_display_get_pointer (gdk_screen_get_display (screen),
+			   &pointer_screen, 
+			   &new_x, &new_y, NULL);
+  if (pointer_screen != screen)
+    {
+      GtkHandleBoxPrivate *private = gtk_handle_box_get_private (hb);
+
+      new_x = private->orig_x;
+      new_y = private->orig_y;
+    }
+  
   new_x += hb->float_allocation.x;
   new_y += hb->float_allocation.y;
 
   snap_edge = hb->snap_edge;
   if (snap_edge == -1)
-    snap_edge = (hb->handle_position == GTK_POS_LEFT ||
-		  hb->handle_position == GTK_POS_RIGHT) ?
+    snap_edge = (handle_position == GTK_POS_LEFT ||
+		 handle_position == GTK_POS_RIGHT) ?
       GTK_POS_TOP : GTK_POS_LEFT;
 
+  if (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) 
+    switch (snap_edge) 
+      {
+      case GTK_POS_LEFT:
+	snap_edge = GTK_POS_RIGHT;
+	break;
+      case GTK_POS_RIGHT:
+	snap_edge = GTK_POS_LEFT;
+	break;
+      default:
+	break;
+      }
+
   /* First, check if the snapped edge is aligned
    */
   switch (snap_edge)
@@ -1122,9 +1240,10 @@ gtk_handle_box_motion (GtkWidget      *widget,
 	  gdk_window_hide (hb->float_window);
 	  gdk_window_reparent (hb->bin_window, widget->window, 0, 0);
 	  hb->float_window_mapped = FALSE;
-	  gtk_signal_emit (GTK_OBJECT (hb),
-			   handle_box_signals[SIGNAL_CHILD_ATTACHED],
-			   GTK_BIN (hb)->child);
+	  g_signal_emit (hb,
+			 handle_box_signals[SIGNAL_CHILD_ATTACHED],
+			 0,
+			 GTK_BIN (hb)->child);
 	  
 	  gtk_widget_queue_resize (widget);
 	}
@@ -1133,11 +1252,11 @@ gtk_handle_box_motion (GtkWidget      *widget,
     {
       gint width, height;
 
-      gdk_window_get_size (hb->float_window, &width, &height);
+      gdk_drawable_get_size (hb->float_window, &width, &height);
       new_x += hb->deskoff_x;
       new_y += hb->deskoff_y;
 
-      switch (hb->handle_position)
+      switch (handle_position)
 	{
 	case GTK_POS_LEFT:
 	  new_y += ((gint)hb->float_allocation.height - height) / 2;
@@ -1179,14 +1298,14 @@ gtk_handle_box_motion (GtkWidget      *widget,
 	  width = child_requisition.width + 2 * GTK_CONTAINER (hb)->border_width;
 	  height = child_requisition.height + 2 * GTK_CONTAINER (hb)->border_width;
 
-	  if (hb->handle_position == GTK_POS_LEFT || hb->handle_position == GTK_POS_RIGHT)
+	  if (handle_position == GTK_POS_LEFT || handle_position == GTK_POS_RIGHT)
 	    width += DRAG_HANDLE_SIZE;
 	  else
 	    height += DRAG_HANDLE_SIZE;
 	  
 	  gdk_window_move_resize (hb->float_window, new_x, new_y, width, height);
 	  gdk_window_reparent (hb->bin_window, hb->float_window, 0, 0);
-	  gdk_window_set_hints (hb->float_window, new_x, new_y, 0, 0, 0, 0, GDK_HINT_POS);
+	  gdk_window_set_geometry_hints (hb->float_window, &geometry, GDK_HINT_POS);
 	  gdk_window_show (hb->float_window);
 	  hb->float_window_mapped = TRUE;
 #if	0
@@ -1197,9 +1316,10 @@ gtk_handle_box_motion (GtkWidget      *widget,
 	  gdk_window_move (hb->float_window, new_x, new_y);
 	  gdk_display_sync (gtk_widget_get_display (widget));
 #endif	/* 0 */
-	  gtk_signal_emit (GTK_OBJECT (hb),
-			   handle_box_signals[SIGNAL_CHILD_DETACHED],
-			   GTK_BIN (hb)->child);
+	  g_signal_emit (hb,
+			 handle_box_signals[SIGNAL_CHILD_DETACHED],
+			 0,
+			 GTK_BIN (hb)->child);
 	  gtk_handle_box_draw_ghost (hb);
 	  
 	  gtk_widget_queue_resize (widget);
@@ -1256,9 +1376,10 @@ gtk_handle_box_reattach (GtkHandleBox *hb)
 	  gdk_window_reparent (hb->bin_window, widget->window, 0, 0);
 
 	  if (GTK_BIN (hb)->child)
-	    gtk_signal_emit (GTK_OBJECT (hb),
-			     handle_box_signals[SIGNAL_CHILD_ATTACHED],
-			     GTK_BIN (hb)->child);
+	    g_signal_emit (hb,
+			   handle_box_signals[SIGNAL_CHILD_ATTACHED],
+			   0,
+			   GTK_BIN (hb)->child);
 
 	}
       hb->float_window_mapped = FALSE;
