@@ -48,6 +48,9 @@
 
 #include <pango/pango-utils.h>	/* For pango_split_file_list */
 
+#include "config.h"
+#include "gtkintl.h"
+
 #include "gtkaccelmap.h"
 #include "gtkbox.h"
 #include "gtkdnd.h"
@@ -59,9 +62,7 @@
 #include "gtkwidget.h"
 #include "gtkwindow.h"
 #include "gtkprivate.h"
-#include "config.h"
 #include "gtkdebug.h"
-#include "gtkintl.h"
 
 /* Private type definitions
  */
@@ -1351,6 +1352,12 @@ gtk_main_do_event (GdkEvent *event)
   if (next_event)
     gdk_event_free (next_event);
 
+  if (event->type == GDK_SETTING)
+    {
+      _gtk_settings_handle_event (&event->setting);
+      return;
+    }
+
   /* Find the widget which got the event. We store the widget
    *  in the user_data field of GdkWindow's.
    *  Ignore the event if we don't have a widget for it, except
@@ -1370,8 +1377,6 @@ gtk_main_do_event (GdkEvent *event)
       if (event->type == GDK_PROPERTY_NOTIFY)
 	_gtk_selection_incr_event (event->any.window,
 				   &event->property);
-      else if (event->type == GDK_SETTING)
-	_gtk_settings_handle_event (&event->setting);
 
       return;
     }
