@@ -33,13 +33,10 @@
 #include "gtkmenu.h"
 #include "gtkmenubar.h"
 #include "gtkmenuitem.h"
-#include "gtktearoffmenuitem.h"
 #include "gtkseparatormenuitem.h"
-#include "gtksignal.h"
 
 
 #define BORDER_SPACING  3
-#define SELECT_TIMEOUT  75
 
 #define MENU_ITEM_CLASS(w)  GTK_MENU_ITEM_CLASS (GTK_OBJECT (w)->klass)
 
@@ -100,14 +97,11 @@ static void gtk_menu_item_forall         (GtkContainer    *container,
 
 static GtkItemClass *parent_class;
 static guint menu_item_signals[LAST_SIGNAL] = { 0 };
-static guint32	last_submenu_deselect_time = 0;
 
-
-
-GtkType
+GType
 gtk_menu_item_get_type (void)
 {
-  static GtkType menu_item_type = 0;
+  static GType menu_item_type = 0;
 
   if (!menu_item_type)
     {
@@ -124,7 +118,8 @@ gtk_menu_item_get_type (void)
 	(GInstanceInitFunc) gtk_menu_item_init,
       };
 
-      menu_item_type = g_type_register_static (GTK_TYPE_ITEM, "GtkMenuItem", &menu_item_info, 0);
+      menu_item_type = g_type_register_static (GTK_TYPE_ITEM, "GtkMenuItem",
+					       &menu_item_info, 0);
     }
 
   return menu_item_type;
@@ -170,39 +165,52 @@ gtk_menu_item_class_init (GtkMenuItemClass *klass)
   klass->hide_on_activate = TRUE;
 
   menu_item_signals[ACTIVATE] =
-    gtk_signal_new ("activate",
-                    GTK_RUN_FIRST | GTK_RUN_ACTION,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuItemClass, activate),
-                    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("activate",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
+		  G_STRUCT_OFFSET (GtkMenuItemClass, activate),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   widget_class->activate_signal = menu_item_signals[ACTIVATE];
 
   menu_item_signals[ACTIVATE_ITEM] =
-    gtk_signal_new ("activate_item",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuItemClass, activate_item),
-                    gtk_signal_default_marshaller,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("activate_item",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkMenuItemClass, activate_item),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
 
   menu_item_signals[TOGGLE_SIZE_REQUEST] =
-    gtk_signal_new ("toggle_size_request",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuItemClass, toggle_size_request),
-                    _gtk_marshal_VOID__POINTER,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_POINTER);
+    g_signal_new ("toggle_size_request",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkMenuItemClass, toggle_size_request),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__POINTER,
+		  G_TYPE_NONE, 1,
+		  G_TYPE_POINTER);
 
   menu_item_signals[TOGGLE_SIZE_ALLOCATE] =
-    gtk_signal_new ("toggle_size_allocate",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkMenuItemClass, toggle_size_allocate),
-                    _gtk_marshal_NONE__INT,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_INT);
+    g_signal_new ("toggle_size_allocate",
+		  G_OBJECT_CLASS_TYPE (gobject_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkMenuItemClass, toggle_size_allocate),
+		  NULL, NULL,
+		  _gtk_marshal_NONE__INT,
+		  G_TYPE_NONE, 1,
+		  G_TYPE_INT);
+
+  gtk_widget_class_install_style_property_parser (widget_class,
+						  g_param_spec_enum ("selected_shadow_type",
+								     "Selected Shadow Type",
+								     "Shadow type when item is selected",
+								     GTK_TYPE_SHADOW_TYPE,
+								     GTK_SHADOW_NONE,
+								     G_PARAM_READABLE),
+						  gtk_rc_property_parse_enum);
 }
 
 static void
@@ -214,7 +222,10 @@ gtk_menu_item_init (GtkMenuItem *menu_item)
   menu_item->toggle_size = 0;
   menu_item->accelerator_width = 0;
   menu_item->show_submenu_indicator = FALSE;
-  menu_item->submenu_direction = GTK_DIRECTION_RIGHT;
+  if (gtk_widget_get_direction (GTK_WIDGET (menu_item)) == GTK_TEXT_DIR_RTL)
+    menu_item->submenu_direction = GTK_DIRECTION_LEFT;
+  else
+    menu_item->submenu_direction = GTK_DIRECTION_RIGHT;
   menu_item->submenu_placement = GTK_TOP_BOTTOM;
   menu_item->right_justify = FALSE;
 
@@ -224,7 +235,7 @@ gtk_menu_item_init (GtkMenuItem *menu_item)
 GtkWidget*
 gtk_menu_item_new (void)
 {
-  return GTK_WIDGET (gtk_type_new (gtk_menu_item_get_type ()));
+  return g_object_new (GTK_TYPE_MENU_ITEM, NULL);
 }
 
 GtkWidget*
@@ -262,7 +273,7 @@ gtk_menu_item_new_with_mnemonic (const gchar *label)
   GtkWidget *accel_label;
 
   menu_item = gtk_menu_item_new ();
-  accel_label = gtk_type_new (GTK_TYPE_ACCEL_LABEL);
+  accel_label = g_object_new (GTK_TYPE_ACCEL_LABEL, NULL);
   gtk_label_set_text_with_mnemonic (GTK_LABEL (accel_label), label);
   gtk_misc_set_alignment (GTK_MISC (accel_label), 0.0, 0.5);
 
@@ -391,7 +402,7 @@ gtk_menu_item_activate (GtkMenuItem *menu_item)
 {
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
   
-  gtk_signal_emit (GTK_OBJECT (menu_item), menu_item_signals[ACTIVATE]);
+  g_signal_emit (menu_item, menu_item_signals[ACTIVATE], 0);
 }
 
 void
@@ -400,7 +411,7 @@ gtk_menu_item_toggle_size_request (GtkMenuItem *menu_item,
 {
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
-  gtk_signal_emit (GTK_OBJECT (menu_item), menu_item_signals[TOGGLE_SIZE_REQUEST], requisition);
+  g_signal_emit (menu_item, menu_item_signals[TOGGLE_SIZE_REQUEST], 0, requisition);
 }
 
 void
@@ -409,7 +420,7 @@ gtk_menu_item_toggle_size_allocate (GtkMenuItem *menu_item,
 {
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
 
-  gtk_signal_emit (GTK_OBJECT (menu_item), menu_item_signals[TOGGLE_SIZE_ALLOCATE], allocation);
+  g_signal_emit (menu_item, menu_item_signals[TOGGLE_SIZE_ALLOCATE], 0, allocation);
 }
 
 static void
@@ -459,10 +470,10 @@ gtk_menu_item_size_request (GtkWidget      *widget,
 
       requisition->width += child_requisition.width;
       requisition->height += child_requisition.height;
-    }
 
-  if (menu_item->submenu && menu_item->show_submenu_indicator)
-    requisition->width += 21;
+      if (menu_item->submenu && menu_item->show_submenu_indicator)
+	requisition->width += child_requisition.height;
+    }
 
   accel_width = 0;
   gtk_container_foreach (GTK_CONTAINER (menu_item),
@@ -478,6 +489,7 @@ gtk_menu_item_size_allocate (GtkWidget     *widget,
   GtkMenuItem *menu_item;
   GtkBin *bin;
   GtkAllocation child_allocation;
+  GtkTextDirection direction;
 
   g_return_if_fail (GTK_IS_MENU_ITEM (widget));
   g_return_if_fail (allocation != NULL);
@@ -485,10 +497,14 @@ gtk_menu_item_size_allocate (GtkWidget     *widget,
   menu_item = GTK_MENU_ITEM (widget);
   bin = GTK_BIN (widget);
   
+  direction = gtk_widget_get_direction (widget);
+
   widget->allocation = *allocation;
 
   if (bin->child)
     {
+      GtkRequisition child_requisition;
+      
       child_allocation.x = (GTK_CONTAINER (widget)->border_width +
                             widget->style->xthickness +
 			    BORDER_SPACING);
@@ -496,14 +512,23 @@ gtk_menu_item_size_allocate (GtkWidget     *widget,
 			    widget->style->ythickness);
       child_allocation.width = MAX (1, (gint)allocation->width - child_allocation.x * 2);
       child_allocation.height = MAX (1, (gint)allocation->height - child_allocation.y * 2);
-      child_allocation.x += GTK_MENU_ITEM (widget)->toggle_size;
+      if (direction == GTK_TEXT_DIR_LTR)
+	child_allocation.x += GTK_MENU_ITEM (widget)->toggle_size;
       child_allocation.width -= GTK_MENU_ITEM (widget)->toggle_size;
       child_allocation.x += widget->allocation.x;
       child_allocation.y += widget->allocation.y;
+
+      gtk_widget_get_child_requisition (bin->child, &child_requisition);
+      if (menu_item->submenu && menu_item->show_submenu_indicator) 
+	{
+	  if (direction == GTK_TEXT_DIR_RTL)
+	    child_allocation.x += child_requisition.height;
+	  child_allocation.width -= child_requisition.height;
+	}
       
-      if (menu_item->submenu && menu_item->show_submenu_indicator)
-	child_allocation.width -= 21;
-      
+      if (child_allocation.width < 1)
+	child_allocation.width = 1;
+
       gtk_widget_size_allocate (bin->child, &child_allocation);
     }
 
@@ -526,7 +551,7 @@ gtk_menu_item_realize (GtkWidget *widget)
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
 
   widget->window = gtk_widget_get_parent_window (widget);
-  gdk_window_ref (widget->window);
+  g_object_ref (widget->window);
   
   attributes.x = widget->allocation.x;
   attributes.y = widget->allocation.y;
@@ -588,7 +613,7 @@ gtk_menu_item_paint (GtkWidget    *widget,
 {
   GtkMenuItem *menu_item;
   GtkStateType state_type;
-  GtkShadowType shadow_type;
+  GtkShadowType shadow_type, selected_shadow_type;
   gint width, height;
   gint x, y;
   gint border_width = GTK_CONTAINER (widget)->border_width;
@@ -598,7 +623,7 @@ gtk_menu_item_paint (GtkWidget    *widget,
       menu_item = GTK_MENU_ITEM (widget);
 
       state_type = widget->state;
-
+      
       x = widget->allocation.x + border_width;
       y = widget->allocation.y + border_width;
       width = widget->allocation.width - border_width * 2;
@@ -606,31 +631,63 @@ gtk_menu_item_paint (GtkWidget    *widget,
       
       if ((state_type == GTK_STATE_PRELIGHT) &&
 	  (GTK_BIN (menu_item)->child))
-	gtk_paint_box (widget->style,
-		       widget->window,
-		       GTK_STATE_PRELIGHT,
-		       GTK_SHADOW_OUT,
-		       area, widget, "menuitem",
-		       x, y, width, height);
-
+	{
+	  gtk_widget_style_get (widget,
+				"selected_shadow_type", &selected_shadow_type,
+				NULL);
+	  gtk_paint_box (widget->style,
+			 widget->window,
+			 GTK_STATE_PRELIGHT,
+			 selected_shadow_type,
+			 area, widget, "menuitem",
+			 x, y, width, height);
+	}
+  
       if (menu_item->submenu && menu_item->show_submenu_indicator)
 	{
+	  GtkRequisition child_requisition;
+	  gint arrow_x, arrow_y;
+	  gint arrow_size;
+	  gint arrow_extent;
+	  GtkTextDirection direction;
+	  GtkArrowType arrow_type;
+
+	  direction = gtk_widget_get_direction (widget);
+      
+	  gtk_widget_get_child_requisition (GTK_BIN (menu_item)->child,
+					    &child_requisition);
+
+	  arrow_size = child_requisition.height - 2 * widget->style->ythickness;
+	  arrow_extent = arrow_size * 0.6;
+	  
 	  shadow_type = GTK_SHADOW_OUT;
 	  if (state_type == GTK_STATE_PRELIGHT)
 	    shadow_type = GTK_SHADOW_IN;
 
+	  if (direction == GTK_TEXT_DIR_LTR) {
+	    arrow_x = x + width - 1 - arrow_size + (arrow_size - arrow_extent) / 2;
+	    arrow_type = GTK_ARROW_RIGHT;
+	  }
+	  else {
+	    arrow_x = x + 1 + (arrow_size - arrow_extent) / 2;
+	    arrow_type = GTK_ARROW_LEFT;
+	  }
+
+	  arrow_y = y + (height - arrow_extent) / 2;
+
 	  gtk_paint_arrow (widget->style, widget->window,
 			   state_type, shadow_type, 
 			   area, widget, "menuitem", 
-			   GTK_ARROW_RIGHT, TRUE,
-			   x + width - 15, y + height / 2 - 5, 10, 10);
+			   arrow_type, TRUE,
+			   arrow_x, arrow_y,
+			   arrow_extent, arrow_extent);
 	}
       else if (!GTK_BIN (menu_item)->child)
 	{
-	   gtk_paint_hline (widget->style, widget->window, GTK_STATE_NORMAL,
-			    area, widget, "menuitem",
-			    widget->allocation.x, widget->allocation.width,
-			    widget->allocation.y);
+	  gtk_paint_hline (widget->style, widget->window, GTK_STATE_NORMAL,
+			   area, widget, "menuitem",
+			   widget->allocation.x, widget->allocation.width,
+			   widget->allocation.y);
 	}
     }
 }
@@ -652,6 +709,27 @@ gtk_menu_item_expose (GtkWidget      *widget,
   return FALSE;
 }
 
+static gint
+get_popup_delay (GtkMenuItem *menu_item)
+{
+  GtkWidget *parent = GTK_WIDGET (menu_item)->parent;
+
+  if (GTK_IS_MENU_SHELL (parent))
+    {
+      return _gtk_menu_shell_get_popup_delay (GTK_MENU_SHELL (parent));
+    }
+  else
+    {
+      gint popup_delay;
+      
+      g_object_get (G_OBJECT (gtk_widget_get_settings (GTK_WIDGET (menu_item))),
+		    "gtk-menu-popup-delay", &popup_delay,
+		    NULL);
+
+      return popup_delay;
+    }
+}
+
 static void
 gtk_real_menu_item_select (GtkItem *item)
 {
@@ -661,26 +739,38 @@ gtk_real_menu_item_select (GtkItem *item)
 
   menu_item = GTK_MENU_ITEM (item);
 
-  /*  if (menu_item->submenu && !GTK_WIDGET_VISIBLE (menu_item->submenu))*/
   if (menu_item->submenu)
     {
-      guint32 etime;
-      GdkEvent *event = gtk_get_current_event ();
+      gint popup_delay;
 
-      etime = event ? gdk_event_get_time (event) : GDK_CURRENT_TIME;
-      if (etime >= last_submenu_deselect_time &&
-	  last_submenu_deselect_time + SELECT_TIMEOUT > etime)
-	menu_item->timer = gtk_timeout_add (SELECT_TIMEOUT - (etime - last_submenu_deselect_time),
-					    gtk_menu_item_select_timeout,
-					    menu_item);
+      if (menu_item->timer)
+	gtk_timeout_remove (menu_item->timer);
+
+      popup_delay = get_popup_delay (menu_item);
+      
+      if (popup_delay > 0)
+	{
+	  GdkEvent *event = gtk_get_current_event ();
+	  
+	  menu_item->timer = gtk_timeout_add (popup_delay,
+					      gtk_menu_item_select_timeout,
+					      menu_item);
+	  if (event &&
+	      event->type != GDK_BUTTON_PRESS &&
+	      event->type != GDK_ENTER_NOTIFY)
+	    menu_item->timer_from_keypress = TRUE;
+	  else
+	    menu_item->timer_from_keypress = FALSE;
+
+	  if (event)
+	    gdk_event_free (event);
+	}
       else
 	gtk_menu_item_popup_submenu (menu_item);
-      if (event)
-	gdk_event_free(event);
     }
   
   gtk_widget_set_state (GTK_WIDGET (menu_item), GTK_STATE_PRELIGHT);
-  gtk_widget_draw (GTK_WIDGET (menu_item), NULL);
+  gtk_widget_queue_draw (GTK_WIDGET (menu_item));
 }
 
 static void
@@ -694,9 +784,6 @@ gtk_real_menu_item_deselect (GtkItem *item)
 
   if (menu_item->submenu)
     {
-      guint32 etime;
-      GdkEvent *event = gtk_get_current_event ();
-
       if (menu_item->timer)
 	{
 	  gtk_timeout_remove (menu_item->timer);
@@ -704,32 +791,26 @@ gtk_real_menu_item_deselect (GtkItem *item)
 	}
       else
 	gtk_menu_popdown (GTK_MENU (menu_item->submenu));
-
-      etime = event ? gdk_event_get_time (event) : GDK_CURRENT_TIME;
-      if (etime > last_submenu_deselect_time)
-	last_submenu_deselect_time = etime;
-      if (event)
-	gdk_event_free(event);
     }
 
   gtk_widget_set_state (GTK_WIDGET (menu_item), GTK_STATE_NORMAL);
-  gtk_widget_draw (GTK_WIDGET (menu_item), NULL);
+  gtk_widget_queue_draw (GTK_WIDGET (menu_item));
 }
 
 static gboolean
 gtk_menu_item_mnemonic_activate (GtkWidget *widget,
 				 gboolean   group_cycling)
 {
-  if (group_cycling)
+  if (group_cycling &&
+      widget->parent &&
+      GTK_IS_MENU_SHELL (widget->parent) &&
+      GTK_MENU_SHELL (widget->parent)->active)
     {
-      if (widget->parent &&
-	  GTK_IS_MENU_SHELL (widget->parent))
-	gtk_menu_shell_select_item (GTK_MENU_SHELL (widget->parent),
-				    widget);
-
+      gtk_menu_shell_select_item (GTK_MENU_SHELL (widget->parent),
+				  widget);
     }
   else
-    gtk_signal_emit (GTK_OBJECT (widget), menu_item_signals[ACTIVATE_ITEM]);
+    g_signal_emit (widget, menu_item_signals[ACTIVATE_ITEM], 0);
   
   return TRUE;
 }
@@ -755,25 +836,13 @@ gtk_real_menu_item_activate_item (GtkMenuItem *menu_item)
 	{
 	  GtkMenuShell *menu_shell = GTK_MENU_SHELL (widget->parent);
 
-	  if (!menu_shell->active)
-	    {
-	      gtk_grab_add (GTK_WIDGET (menu_shell));
-	      menu_shell->have_grab = TRUE;
-	      menu_shell->active = TRUE;
-	    }
+	  _gtk_menu_shell_activate (menu_shell);
 
 	  gtk_menu_shell_select_item (GTK_MENU_SHELL (widget->parent), widget); 
 	  gtk_menu_item_popup_submenu (widget); 
 
+	  gtk_menu_shell_select_first (GTK_MENU_SHELL (menu_item->submenu), TRUE);
 	  submenu = GTK_MENU_SHELL (menu_item->submenu);
-	  if (submenu->children)
-	    {
-	      if (submenu->children->next &&
-		  GTK_IS_TEAROFF_MENU_ITEM (submenu->children->data))
-		gtk_menu_shell_select_item (submenu, submenu->children->next->data);
-	      else
-		gtk_menu_shell_select_item (submenu, submenu->children->data);
-	    }
 	}
     }
 }
@@ -798,9 +867,21 @@ gtk_real_menu_item_toggle_size_allocate (GtkMenuItem *menu_item,
 static gint
 gtk_menu_item_select_timeout (gpointer data)
 {
+  GtkMenuItem *menu_item;
+  GtkWidget *parent;
+  
   GDK_THREADS_ENTER ();
 
-  gtk_menu_item_popup_submenu (data);
+  menu_item = GTK_MENU_ITEM (data);
+
+  parent = GTK_WIDGET (menu_item)->parent;
+
+  if (parent && GTK_IS_MENU_SHELL (parent) && GTK_MENU_SHELL (parent)->active)
+    {
+      gtk_menu_item_popup_submenu (data);
+      if (menu_item->timer_from_keypress && menu_item->submenu)
+	GTK_MENU_SHELL (menu_item->submenu)->ignore_enter = TRUE;
+    }
 
   GDK_THREADS_LEAVE ();
 
@@ -810,21 +891,24 @@ gtk_menu_item_select_timeout (gpointer data)
 static void
 gtk_menu_item_popup_submenu (gpointer data)
 {
+  GtkWidget *widget;
   GtkMenuItem *menu_item;
 
-  menu_item = GTK_MENU_ITEM (data);
+  widget = GTK_WIDGET (data);
+  menu_item = GTK_MENU_ITEM (widget);
+
+  if (menu_item->timer)
+    gtk_timeout_remove (menu_item->timer);
   menu_item->timer = 0;
 
   if (GTK_WIDGET_IS_SENSITIVE (menu_item->submenu))
-    {
-      gtk_menu_popup (GTK_MENU (menu_item->submenu),
-		      GTK_WIDGET (menu_item)->parent,
-		      GTK_WIDGET (menu_item),
-		      gtk_menu_item_position_menu,
-		      menu_item,
-		      GTK_MENU_SHELL (GTK_WIDGET (menu_item)->parent)->button,
-		      0);
-    }
+    gtk_menu_popup (GTK_MENU (menu_item->submenu),
+		    widget->parent,
+		    widget,
+		    gtk_menu_item_position_menu,
+		    menu_item,
+		    GTK_MENU_SHELL (widget->parent)->button,
+		    0);
 }
 
 static void
@@ -837,10 +921,12 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
   GtkMenuItem *menu_item;
   GtkWidget *widget;
   GtkWidget *parent_menu_item;
-  gint screen_width;
-  gint screen_height;
+  GdkScreen *screen;
   gint twidth, theight;
   gint tx, ty;
+  GtkTextDirection direction;
+  GdkRectangle monitor;
+  gint monitor_num;
 
   g_return_if_fail (menu != NULL);
   g_return_if_fail (x != NULL);
@@ -849,11 +935,16 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
   menu_item = GTK_MENU_ITEM (user_data);
   widget = GTK_WIDGET (user_data);
 
+  direction = gtk_widget_get_direction (widget);
+
   twidth = GTK_WIDGET (menu)->requisition.width;
   theight = GTK_WIDGET (menu)->requisition.height;
 
-  screen_width = gdk_screen_width ();
-  screen_height = gdk_screen_height ();
+  screen = gtk_widget_get_screen (widget);
+  monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+  if (monitor_num < 0)
+    monitor_num = 0;
+  gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
   if (!gdk_window_get_origin (widget->window, &tx, &ty))
     {
@@ -867,26 +958,39 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
   switch (menu_item->submenu_placement)
     {
     case GTK_TOP_BOTTOM:
-      if ((ty + widget->allocation.height + theight) <= screen_height)
+      if (direction == GTK_TEXT_DIR_LTR)
+	menu_item->submenu_direction = GTK_DIRECTION_RIGHT;
+      else 
+	{
+	  menu_item->submenu_direction = GTK_DIRECTION_LEFT;
+	  tx += widget->allocation.width - twidth;
+	}
+
+      if ((ty + widget->allocation.height + theight) <= monitor.y + monitor.height)
 	ty += widget->allocation.height;
-      else if ((ty - theight) >= 0)
+      else if ((ty - theight) >= monitor.y)
 	ty -= theight;
-      else if (screen_height - (ty + widget->allocation.height) > ty)
+      else if (monitor.y + monitor.height - (ty + widget->allocation.height) > ty)
 	ty += widget->allocation.height;
       else
 	ty -= theight;
       break;
 
     case GTK_LEFT_RIGHT:
-      menu_item->submenu_direction = GTK_DIRECTION_RIGHT;
       parent_menu_item = GTK_MENU (widget->parent)->parent_menu_item;
-      if (parent_menu_item)
+      if (parent_menu_item && 
+	  !GTK_MENU (widget->parent)->torn_off && 
+	  !GTK_MENU_SHELL (menu)->active)
 	menu_item->submenu_direction = GTK_MENU_ITEM (parent_menu_item)->submenu_direction;
+      else if (direction == GTK_TEXT_DIR_LTR)
+	menu_item->submenu_direction = GTK_DIRECTION_RIGHT;
+      else
+	menu_item->submenu_direction = GTK_DIRECTION_LEFT;
 
       switch (menu_item->submenu_direction)
 	{
 	case GTK_DIRECTION_LEFT:
-	  if ((tx - twidth) >= 0)
+	  if ((tx - twidth) >= monitor.x)
 	    tx -= twidth;
 	  else
 	    {
@@ -896,7 +1000,7 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
 	  break;
 
 	case GTK_DIRECTION_RIGHT:
-	  if ((tx + widget->allocation.width + twidth - 5) <= screen_width)
+	  if ((tx + widget->allocation.width + twidth - 5) <= monitor.x + monitor.width)
 	    tx += widget->allocation.width - 5;
 	  else
 	    {
@@ -909,14 +1013,14 @@ gtk_menu_item_position_menu (GtkMenu  *menu,
       ty += widget->allocation.height / 4;
 
       /* If the height of the menu doesn't fit we move it upward. */
-      ty = CLAMP (ty, 0, MAX (0, screen_height - theight));
+      ty = CLAMP (ty, monitor.y, MAX (monitor.y, monitor.y + monitor.height - theight));
       break;
     }
 
   /* If we have negative, tx, here it is because we can't get
    * the menu all the way on screen. Favor the left portion.
    */
-  *x = CLAMP (tx, 0, MAX (0, screen_width - twidth));
+  *x = CLAMP (tx, monitor.x, MAX (monitor.x, monitor.x + monitor.width - twidth));
   *y = ty;
 }
 
@@ -1058,7 +1162,7 @@ _gtk_menu_item_refresh_accel_path (GtkMenuItem   *menu_item,
       return;
     }
 
-  path = _gtk_widget_get_accel_path (widget);
+  path = _gtk_widget_get_accel_path (widget, NULL);
   if (!path)					/* no active accel_path yet */
     {
       path = menu_item->accel_path;
@@ -1083,7 +1187,8 @@ _gtk_menu_item_refresh_accel_path (GtkMenuItem   *menu_item,
 /**
  * gtk_menu_item_set_accel_path
  * @menu_item:  a valid #GtkMenuItem
- * @accel_path: accelerator path, corresponding to this menu item's functionality
+ * @accel_path: accelerator path, corresponding to this menu item's
+ *              functionality, or %NULL to unset the current path.
  *
  * Set the accelerator path on @menu_item, through which runtime changes of the
  * menu item's accelerator caused by the user can be identified and saved to
@@ -1096,6 +1201,9 @@ _gtk_menu_item_refresh_accel_path (GtkMenuItem   *menu_item,
  * This function is basically a convenience wrapper that handles calling
  * gtk_widget_set_accel_path() with the appropriate accelerator group for
  * the menu item.
+ *
+ * Note that you do need to set an accelerator on the parent menu with
+ * gtk_menu_set_accel_group() for this to work.
  */
 void
 gtk_menu_item_set_accel_path (GtkMenuItem *menu_item,
@@ -1104,7 +1212,8 @@ gtk_menu_item_set_accel_path (GtkMenuItem *menu_item,
   GtkWidget *widget;
 
   g_return_if_fail (GTK_IS_MENU_ITEM (menu_item));
-  g_return_if_fail (accel_path && accel_path[0] == '<' && strchr (accel_path, '/'));
+  g_return_if_fail (accel_path == NULL ||
+		    (accel_path[0] == '<' && strchr (accel_path, '/')));
 
   widget = GTK_WIDGET (menu_item);
 
@@ -1116,7 +1225,7 @@ gtk_menu_item_set_accel_path (GtkMenuItem *menu_item,
   gtk_widget_set_accel_path (widget, NULL, NULL);
 
   /* install accelerators associated with new path */
-  if (widget->parent)
+  if (widget->parent && GTK_IS_MENU (widget->parent))
     {
       GtkMenu *menu = GTK_MENU (widget->parent);
 
