@@ -77,8 +77,7 @@ gtk_text_mark_get_type (void)
         (GInstanceInitFunc) gtk_text_mark_init,
       };
 
-      object_type = g_type_register_static (G_TYPE_OBJECT,
-                                            "GtkTextMark",
+      object_type = g_type_register_static (G_TYPE_OBJECT, "GtkTextMark",
                                             &object_info, 0);
     }
 
