@@ -2,34 +2,39 @@
  * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
 /*
- * Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
+ * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include "gtktree.h"
-#include "gtktreeitem.h"
+#undef GTK_DISABLE_DEPRECATED
+
 #include "gtkmain.h"
+#include "gtkmarshalers.h"
 #include "gtksignal.h"
 #include "gtklist.h"
 
+#define GTK_ENABLE_BROKEN
+#include "gtktree.h"
+#include "gtktreeitem.h"
+
 enum {
   SELECTION_CHANGED,
   SELECT_CHILD,
@@ -45,10 +50,6 @@ static void gtk_tree_parent_set      (GtkWidget      *widget,
 				      GtkWidget      *previous_parent);
 static void gtk_tree_unmap           (GtkWidget      *widget);
 static void gtk_tree_realize         (GtkWidget      *widget);
-static void gtk_tree_draw            (GtkWidget      *widget,
-				      GdkRectangle   *area);
-static gint gtk_tree_expose          (GtkWidget      *widget,
-				      GdkEventExpose *event);
 static gint gtk_tree_motion_notify   (GtkWidget      *widget,
 				      GdkEventMotion *event);
 static gint gtk_tree_button_press    (GtkWidget      *widget,
@@ -114,31 +115,6 @@ gtk_tree_class_init (GtkTreeClass *class)
   
   parent_class = gtk_type_class (gtk_container_get_type ());
   
-  tree_signals[SELECTION_CHANGED] =
-    gtk_signal_new ("selection_changed",
-		    GTK_RUN_FIRST,
-		    object_class->type,
-		    GTK_SIGNAL_OFFSET (GtkTreeClass, selection_changed),
-		    gtk_marshal_NONE__NONE,
-		    GTK_TYPE_NONE, 0);
-  tree_signals[SELECT_CHILD] =
-    gtk_signal_new ("select_child",
-		    GTK_RUN_FIRST,
-		    object_class->type,
-		    GTK_SIGNAL_OFFSET (GtkTreeClass, select_child),
-		    gtk_marshal_NONE__POINTER,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_WIDGET);
-  tree_signals[UNSELECT_CHILD] =
-    gtk_signal_new ("unselect_child",
-		    GTK_RUN_FIRST,
-		    object_class->type,
-		    GTK_SIGNAL_OFFSET (GtkTreeClass, unselect_child),
-		    gtk_marshal_NONE__POINTER,
-		    GTK_TYPE_NONE, 1,
-		    GTK_TYPE_WIDGET);
-  
-  gtk_object_class_add_signals (object_class, tree_signals, LAST_SIGNAL);
   
   object_class->destroy = gtk_tree_destroy;
   
@@ -146,8 +122,6 @@ gtk_tree_class_init (GtkTreeClass *class)
   widget_class->unmap = gtk_tree_unmap;
   widget_class->parent_set = gtk_tree_parent_set;
   widget_class->realize = gtk_tree_realize;
-  widget_class->draw = gtk_tree_draw;
-  widget_class->expose_event = gtk_tree_expose;
   widget_class->motion_notify_event = gtk_tree_motion_notify;
   widget_class->button_press_event = gtk_tree_button_press;
   widget_class->button_release_event = gtk_tree_button_release;
@@ -163,6 +137,30 @@ gtk_tree_class_init (GtkTreeClass *class)
   class->selection_changed = NULL;
   class->select_child = gtk_real_tree_select_child;
   class->unselect_child = gtk_real_tree_unselect_child;
+
+  tree_signals[SELECTION_CHANGED] =
+    gtk_signal_new ("selection_changed",
+		    GTK_RUN_FIRST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GtkTreeClass, selection_changed),
+		    _gtk_marshal_VOID__VOID,
+		    GTK_TYPE_NONE, 0);
+  tree_signals[SELECT_CHILD] =
+    gtk_signal_new ("select_child",
+		    GTK_RUN_FIRST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GtkTreeClass, select_child),
+		    _gtk_marshal_VOID__OBJECT,
+		    GTK_TYPE_NONE, 1,
+		    GTK_TYPE_WIDGET);
+  tree_signals[UNSELECT_CHILD] =
+    gtk_signal_new ("unselect_child",
+		    GTK_RUN_FIRST,
+		    GTK_CLASS_TYPE (object_class),
+		    GTK_SIGNAL_OFFSET (GtkTreeClass, unselect_child),
+		    _gtk_marshal_VOID__OBJECT,
+		    GTK_TYPE_NONE, 1,
+		    GTK_TYPE_WIDGET);
 }
 
 static GtkType
@@ -183,7 +181,7 @@ gtk_tree_init (GtkTree *tree)
   tree->current_indent = 0;
   tree->level = 0;
   tree->view_mode = GTK_TREE_VIEW_LINE;
-  tree->view_line = 1;
+  tree->view_line = TRUE;
 }
 
 GtkWidget*
@@ -196,9 +194,7 @@ void
 gtk_tree_append (GtkTree   *tree,
 		 GtkWidget *tree_item)
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   gtk_tree_insert (tree, tree_item, -1);
@@ -208,9 +204,7 @@ void
 gtk_tree_prepend (GtkTree   *tree,
 		  GtkWidget *tree_item)
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   gtk_tree_insert (tree, tree_item, 0);
@@ -223,9 +217,7 @@ gtk_tree_insert (GtkTree   *tree,
 {
   gint nchildren;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   nchildren = g_list_length (tree->children);
@@ -239,17 +231,6 @@ gtk_tree_insert (GtkTree   *tree,
     tree->children = g_list_insert (tree->children, tree_item, position);
   
   gtk_widget_set_parent (tree_item, GTK_WIDGET (tree));
-  
-  if (GTK_WIDGET_REALIZED (tree_item->parent))
-    gtk_widget_realize (tree_item);
-
-  if (GTK_WIDGET_VISIBLE (tree_item->parent) && GTK_WIDGET_VISIBLE (tree_item))
-    {
-      if (GTK_WIDGET_MAPPED (tree_item->parent))
-	gtk_widget_map (tree_item);
-
-      gtk_widget_queue_resize (tree_item);
-    }
 }
 
 static void
@@ -258,7 +239,6 @@ gtk_tree_add (GtkContainer *container,
 {
   GtkTree *tree;
   
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_TREE (container));
   g_return_if_fail (GTK_IS_TREE_ITEM (child));
   
@@ -268,17 +248,6 @@ gtk_tree_add (GtkContainer *container,
   
   gtk_widget_set_parent (child, GTK_WIDGET (container));
   
-  if (GTK_WIDGET_REALIZED (child->parent))
-    gtk_widget_realize (child);
-
-  if (GTK_WIDGET_VISIBLE (child->parent) && GTK_WIDGET_VISIBLE (child))
-    {
-      if (GTK_WIDGET_MAPPED (child->parent))
-	gtk_widget_map (child);
-
-      gtk_widget_queue_resize (child);
-    }
-  
   if (!tree->selection && (tree->selection_mode == GTK_SELECTION_BROWSE))
     gtk_tree_select_child (tree, child);
 }
@@ -290,7 +259,6 @@ gtk_tree_button_press (GtkWidget      *widget,
   GtkTree *tree;
   GtkWidget *item;
   
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
   
@@ -326,7 +294,6 @@ gtk_tree_button_release (GtkWidget      *widget,
   GtkTree *tree;
   GtkWidget *item;
   
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
   
@@ -344,7 +311,6 @@ gtk_tree_child_position (GtkTree   *tree,
   gint pos;
   
   
-  g_return_val_if_fail (tree != NULL, -1);
   g_return_val_if_fail (GTK_IS_TREE (tree), -1);
   g_return_val_if_fail (child != NULL, -1);
   
@@ -375,7 +341,6 @@ gtk_tree_clear_items (GtkTree *tree,
   guint nchildren;
   guint index;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   nchildren = g_list_length (tree->children);
@@ -411,7 +376,6 @@ gtk_tree_destroy (GtkObject *object)
   GtkWidget *child;
   GList *children;
   
-  g_return_if_fail (object != NULL);
   g_return_if_fail (GTK_IS_TREE (object));
   
   tree = GTK_TREE (object);
@@ -444,79 +408,6 @@ gtk_tree_destroy (GtkObject *object)
     (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
-static void
-gtk_tree_draw (GtkWidget    *widget,
-	       GdkRectangle *area)
-{
-  GtkTree *tree;
-  GtkWidget *subtree;
-  GtkWidget *child;
-  GdkRectangle child_area;
-  GList *children;
-  
-  
-  g_return_if_fail (widget != NULL);
-  g_return_if_fail (GTK_IS_TREE (widget));
-  g_return_if_fail (area != NULL);
-  
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      tree = GTK_TREE (widget);
-      
-      children = tree->children;
-      while (children)
-	{
-	  child = children->data;
-	  children = children->next;
-	  
-	  if (gtk_widget_intersect (child, area, &child_area))
-	    gtk_widget_draw (child, &child_area);
-	  
-	  if((subtree = GTK_TREE_ITEM(child)->subtree) &&
-	     GTK_WIDGET_VISIBLE(subtree) &&
-	     gtk_widget_intersect (subtree, area, &child_area))
-	    gtk_widget_draw (subtree, &child_area);
-	}
-    }
-  
-}
-
-static gint
-gtk_tree_expose (GtkWidget      *widget,
-		 GdkEventExpose *event)
-{
-  GtkTree *tree;
-  GtkWidget *child;
-  GdkEventExpose child_event;
-  GList *children;
-  
-  
-  g_return_val_if_fail (widget != NULL, FALSE);
-  g_return_val_if_fail (GTK_IS_TREE (widget), FALSE);
-  g_return_val_if_fail (event != NULL, FALSE);
-  
-  if (GTK_WIDGET_DRAWABLE (widget))
-    {
-      tree = GTK_TREE (widget);
-      
-      child_event = *event;
-      
-      children = tree->children;
-      while (children)
-	{
-	  child = children->data;
-	  children = children->next;
-	  
-	  if (GTK_WIDGET_NO_WINDOW (child) &&
-	      gtk_widget_intersect (child, &event->area, &child_event.area))
-	    gtk_widget_event (child, (GdkEvent*) &child_event);
-	}
-    }
-  
-  
-  return FALSE;
-}
-
 static void
 gtk_tree_forall (GtkContainer *container,
 		 gboolean      include_internals,
@@ -528,7 +419,6 @@ gtk_tree_forall (GtkContainer *container,
   GList *children;
   
   
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_TREE (container));
   g_return_if_fail (callback != NULL);
   
@@ -647,7 +537,6 @@ static gint
 gtk_tree_motion_notify (GtkWidget      *widget,
 			GdkEventMotion *event)
 {
-  g_return_val_if_fail (widget != NULL, FALSE);
   g_return_val_if_fail (GTK_IS_TREE (widget), FALSE);
   g_return_val_if_fail (event != NULL, FALSE);
   
@@ -665,7 +554,6 @@ gtk_tree_realize (GtkWidget *widget)
   gint attributes_mask;
   
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE (widget));
   
   GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
@@ -696,7 +584,6 @@ gtk_tree_remove_item (GtkTree      *container,
 {
   GList *item_list;
   
-  g_return_if_fail (container != NULL);
   g_return_if_fail (GTK_IS_TREE (container));
   g_return_if_fail (widget != NULL);
   g_return_if_fail (container == GTK_TREE (widget->parent));
@@ -733,7 +620,6 @@ gtk_tree_remove_items (GtkTree *tree,
   GtkTree *real_tree;
   GtkTree *root_tree;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
 #ifdef TREE_DEBUG
@@ -923,9 +809,7 @@ void
 gtk_tree_select_child (GtkTree   *tree,
 		       GtkWidget *tree_item)
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   gtk_signal_emit (GTK_OBJECT (tree), tree_signals[SELECT_CHILD], tree_item);
@@ -937,7 +821,6 @@ gtk_tree_select_item (GtkTree   *tree,
 {
   GList *tmp_list;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   tmp_list = g_list_nth (tree->children, item);
@@ -956,7 +839,6 @@ gtk_tree_size_allocate (GtkWidget     *widget,
   GList *children;
   
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE (widget));
   g_return_if_fail (allocation != NULL);
   
@@ -1015,7 +897,6 @@ gtk_tree_size_request (GtkWidget      *widget,
   GtkRequisition child_requisition;
   
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE (widget));
   g_return_if_fail (requisition != NULL);
   
@@ -1061,7 +942,6 @@ static void
 gtk_tree_unmap (GtkWidget *widget)
 {
   
-  g_return_if_fail (widget != NULL);
   g_return_if_fail (GTK_IS_TREE (widget));
   
   GTK_WIDGET_UNSET_FLAGS (widget, GTK_MAPPED);
@@ -1073,9 +953,7 @@ void
 gtk_tree_unselect_child (GtkTree   *tree,
 			 GtkWidget *tree_item)
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (tree_item != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (tree_item));
   
   gtk_signal_emit (GTK_OBJECT (tree), tree_signals[UNSELECT_CHILD], tree_item);
@@ -1087,7 +965,6 @@ gtk_tree_unselect_item (GtkTree *tree,
 {
   GList *tmp_list;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   tmp_list = g_list_nth (tree->children, item);
@@ -1104,9 +981,7 @@ gtk_real_tree_select_child (GtkTree   *tree,
   GList *tmp_list;
   GtkWidget *tmp_item;
   
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (child != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (child));
 
   root_selection = tree->root_tree->selection;
@@ -1195,27 +1070,6 @@ gtk_real_tree_select_child (GtkTree   *tree,
       break;
       
     case GTK_SELECTION_MULTIPLE:
-      if (child->state == GTK_STATE_NORMAL)
-	{
-	  gtk_tree_item_select (GTK_TREE_ITEM (child));
-	  root_selection = g_list_prepend (root_selection, child);
-	  gtk_widget_ref (child);
-	  tree->root_tree->selection = root_selection;
-	  gtk_signal_emit (GTK_OBJECT (tree->root_tree), 
-			   tree_signals[SELECTION_CHANGED]);
-	}
-      else if (child->state == GTK_STATE_SELECTED)
-	{
-	  gtk_tree_item_deselect (GTK_TREE_ITEM (child));
-	  root_selection = g_list_remove (root_selection, child);
-	  gtk_widget_unref (child);
-	  tree->root_tree->selection = root_selection;
-	  gtk_signal_emit (GTK_OBJECT (tree->root_tree), 
-			   tree_signals[SELECTION_CHANGED]);
-	}
-      break;
-      
-    case GTK_SELECTION_EXTENDED:
       break;
     }
 }
@@ -1224,15 +1078,12 @@ static void
 gtk_real_tree_unselect_child (GtkTree   *tree,
 			      GtkWidget *child)
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
-  g_return_if_fail (child != NULL);
   g_return_if_fail (GTK_IS_TREE_ITEM (child));
   
   switch (tree->selection_mode)
     {
     case GTK_SELECTION_SINGLE:
-    case GTK_SELECTION_MULTIPLE:
     case GTK_SELECTION_BROWSE:
       if (child->state == GTK_STATE_SELECTED)
 	{
@@ -1245,7 +1096,7 @@ gtk_real_tree_unselect_child (GtkTree   *tree,
 	}
       break;
       
-    case GTK_SELECTION_EXTENDED:
+    case GTK_SELECTION_MULTIPLE:
       break;
     }
 }
@@ -1254,7 +1105,6 @@ void
 gtk_tree_set_selection_mode (GtkTree       *tree,
 			     GtkSelectionMode mode) 
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   tree->selection_mode = mode;
@@ -1264,7 +1114,6 @@ void
 gtk_tree_set_view_mode (GtkTree       *tree,
 			GtkTreeViewMode mode) 
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   tree->view_mode = mode;
@@ -1272,9 +1121,8 @@ gtk_tree_set_view_mode (GtkTree       *tree,
 
 void
 gtk_tree_set_view_lines (GtkTree       *tree,
-			 guint          flag) 
+			 gboolean	flag) 
 {
-  g_return_if_fail (tree != NULL);
   g_return_if_fail (GTK_IS_TREE (tree));
   
   tree->view_line = flag;
