@@ -1,6 +1,7 @@
 
 /* Generated data (by glib-mkenums) */
 
+#undef GTK_DISABLE_DEPRECATED
 #define GTK_ENABLE_BROKEN
 #include "gtk.h"
 #include "gtkprivate.h"
@@ -179,6 +180,7 @@ gtk_cell_renderer_state_get_type (void)
       { GTK_CELL_RENDERER_PRELIT, "GTK_CELL_RENDERER_PRELIT", "prelit" },
       { GTK_CELL_RENDERER_INSENSITIVE, "GTK_CELL_RENDERER_INSENSITIVE", "insensitive" },
       { GTK_CELL_RENDERER_SORTED, "GTK_CELL_RENDERER_SORTED", "sorted" },
+      { GTK_CELL_RENDERER_FOCUSED, "GTK_CELL_RENDERER_FOCUSED", "focused" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static ("GtkCellRendererState", values);
@@ -216,6 +218,7 @@ gtk_debug_flag_get_type (void)
       { GTK_DEBUG_TREE, "GTK_DEBUG_TREE", "tree" },
       { GTK_DEBUG_UPDATES, "GTK_DEBUG_UPDATES", "updates" },
       { GTK_DEBUG_KEYBINDINGS, "GTK_DEBUG_KEYBINDINGS", "keybindings" },
+      { GTK_DEBUG_MULTIHEAD, "GTK_DEBUG_MULTIHEAD", "multihead" },
       { 0, NULL, NULL }
     };
     etype = g_flags_register_static ("GtkDebugFlag", values);
@@ -1006,6 +1009,36 @@ gtk_sort_type_get_type (void)
   return etype;
 }
 
+GType
+gtk_im_preedit_style_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GTK_IM_PREEDIT_NOTHING, "GTK_IM_PREEDIT_NOTHING", "nothing" },
+      { GTK_IM_PREEDIT_CALLBACK, "GTK_IM_PREEDIT_CALLBACK", "callback" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GtkIMPreeditStyle", values);
+  }
+  return etype;
+}
+
+GType
+gtk_im_status_style_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GTK_IM_STATUS_NOTHING, "GTK_IM_STATUS_NOTHING", "nothing" },
+      { GTK_IM_STATUS_CALLBACK, "GTK_IM_STATUS_CALLBACK", "callback" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GtkIMStatusStyle", values);
+  }
+  return etype;
+}
+
 
 /* enumerations from "gtkimage.h" */
 GType
