@@ -343,10 +343,10 @@ struct _GtkWidgetClass
   void (* selection_get)           (GtkWidget          *widget,
 				    GtkSelectionData   *selection_data,
 				    guint               info,
-				    guint               time);
+				    guint               time_);
   void (* selection_received)      (GtkWidget          *widget,
 				    GtkSelectionData   *selection_data,
-				    guint               time);
+				    guint               time_);
 
   /* Source side drag signals */
   void (* drag_begin)	           (GtkWidget	       *widget,
@@ -357,31 +357,31 @@ struct _GtkWidgetClass
 				    GdkDragContext     *context,
 				    GtkSelectionData   *selection_data,
 				    guint               info,
-				    guint               time);
+				    guint               time_);
   void (* drag_data_delete)        (GtkWidget	       *widget,
 				    GdkDragContext     *context);
 
   /* Target side drag signals */
   void (* drag_leave)	           (GtkWidget	       *widget,
 				    GdkDragContext     *context,
-				    guint               time);
+				    guint               time_);
   gboolean (* drag_motion)         (GtkWidget	       *widget,
 				    GdkDragContext     *context,
 				    gint                x,
 				    gint                y,
-				    guint               time);
+				    guint               time_);
   gboolean (* drag_drop)           (GtkWidget	       *widget,
 				    GdkDragContext     *context,
 				    gint                x,
 				    gint                y,
-				    guint               time);
+				    guint               time_);
   void (* drag_data_received)      (GtkWidget          *widget,
 				    GdkDragContext     *context,
 				    gint                x,
 				    gint                y,
 				    GtkSelectionData   *selection_data,
 				    guint               info,
-				    guint               time);
+				    guint               time_);
 
   /* Signals used only for keybindings */
   gboolean (* popup_menu)          (GtkWidget          *widget);
@@ -398,6 +398,8 @@ struct _GtkWidgetClass
    */
   AtkObject*   (* get_accessible)  (GtkWidget          *widget);
 
+  void (* screen_changed) (GtkWidget *widget,
+			   GdkScreen *previous_screen);
   /* Padding for future expansion */
   void (*_gtk_reserved1) (void);
   void (*_gtk_reserved2) (void);
@@ -406,7 +408,6 @@ struct _GtkWidgetClass
   void (*_gtk_reserved5) (void);
   void (*_gtk_reserved6) (void);
   void (*_gtk_reserved7) (void);
-  void (*_gtk_reserved8) (void);
 };
 
 struct _GtkWidgetAuxInfo
@@ -761,6 +762,8 @@ GtkWidgetAuxInfo *_gtk_widget_get_aux_info                (GtkWidget    *widget,
 							   gboolean      create);
 void              _gtk_widget_propagate_hierarchy_changed (GtkWidget    *widget,
 							   GtkWidget    *previous_toplevel);
+void              _gtk_widget_propagate_screen_changed    (GtkWidget    *widget,
+							   GdkScreen    *previous_screen);
 
 GdkColormap* _gtk_widget_peek_colormap (void);
 
